#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "ARINC424_18.h"

double arinc424_18_parse_lat(const char lat[9]) {
	char buf[10];
	int deg,min,hunsec;
	double result;

	/* Degrees */
	memcpy(buf,&(lat[1]),2);
	buf[2]=0;
	deg=atoi(buf);
	if (deg<0 || deg>90)
		return NAN;

	/* Minutes */
	memcpy(buf,&(lat[3]),2);
	buf[2]=0;
	min=atoi(buf);
	if (min<0 || min>60)
		return NAN;

	/* Hundredths of seconds */
	memcpy(buf,&(lat[5]),4);
	buf[4]=0;
	hunsec=atoi(buf);
	if (hunsec<0 || hunsec>10000)
		return NAN;

	result=deg+min/60.0+hunsec/(3600.0*100.0);

	/* North/South */
	if (lat[0]=='S')
		result=-result;
	else if (lat[0]!='N')
		return NAN;

	return result;
}

double arinc424_18_parse_lon(const char lon[10]) {
	char buf[11];
	int deg,min,hunsec;
	double result;

	/* Degrees */
	memcpy(buf,&(lon[1]),3);
	buf[3]=0;
	deg=atoi(buf);
	if (deg<0 || deg>180)
	{
		return NAN;
	}

	/* Minutes */
	memcpy(buf,&(lon[4]),2);
	buf[2]=0;
	min=atoi(buf);
	if (min<0 || min>60)
	{
		return NAN;
	}

	/* Hundredths of seconds */
	memcpy(buf,&(lon[6]),4);
	buf[4]=0;
	hunsec=atoi(buf);
	if (hunsec<0 || hunsec>10000)
	{
		return NAN;
	}

	result=deg+min/60.0+hunsec/(3600.0*100.0);

	/* North/South */
	if (lon[0]=='W')
	{
		result=-result;
	}
	else if (lon[0]!='E')
	{
		return NAN;
	}

	return result;
}

char *arinc424_18_parse_recordtype(const char* line) {
	static char code[3];

	code[0]=line[4];	/* Section */
	if (code[0]=='P' || code[0]=='H')
		code[1]=line[12];	/* Subsection */
	else
		code[1]=line[5];	/* Subsection */
	code[2]=0;

	return code;
}

unsigned int arinc424_18_parse_freq(const char* freqfield) {
	char freq[8];

	memcpy(freq,freqfield,sizeof(freq)-1);
	freq[7]=0;
	return atoi(freq);
}
