#ifndef ARINC424_18_H_
#define ARINC424_18_H_

/* The use of the term "ARINC" in this document is meant to be descriptive of the
 * specification it implements.  This document is not supported, produced or endorsed
 * by ARINC.  Trademarks are the property of their owners.
 */

/* See the ARINC Specification 424-18 for details.  The specification is not free.
 * This file is deliberately using C-based structures heavily instead of classes and
 * inheritance as this is a prime candidate for becoming a C-based library.
 */

/* Record Types (First char=Section Code, Second char [if any]=Subsection code) */
#define ARINC424_18_RECTYPE_NAVAID_VHF "D "
#define ARINC424_18_RECTYPE_NAVAID_NDB "DB"

#define ARINC424_18_RECTYPE_ENROUTE_WAYPOINT "EA"
#define ARINC424_18_RECTYPE_ENROUTE_AIRWAYMARKER "EM"
#define ARINC424_18_RECTYPE_ENROUTE_HOLDINGPATTERNS "EP"
#define ARINC424_18_RECTYPE_ENROUTE_AIRWAYS "ER"
#define ARINC424_18_RECTYPE_ENROUTE_AIRWAYSRESTRICTIONS "EU"
#define ARINC424_18_RECTYPE_ENROUTE_COMMUNICATIONS "EV"

#define ARINC424_18_RECTYPE_AIRPORT "PA"
#define ARINC424_18_RECTYPE_AIRPORT_GATES "PB"
#define ARINC424_18_RECTYPE_AIRPORT_TERMINALWAYPOINTS "PC"
#define ARINC424_18_RECTYPE_AIRPORT_SID "PD"
#define ARINC424_18_RECTYPE_AIRPORT_STAR "PE"
#define ARINC424_18_RECTYPE_AIRPORT_APPROACHES "PF"
#define ARINC424_18_RECTYPE_AIRPORT_RUNWAY "PG"
#define ARINC424_18_RECTYPE_AIRPORT_LOCALIZERGLIDESLOPE "PI"
#define ARINC424_18_RECTYPE_AIRPORT_MLS "PL"
#define ARINC424_18_RECTYPE_AIRPORT_MARKERLOCALIZER "PM"
#define ARINC424_18_RECTYPE_AIRPORT_MSA "PS"
#define ARINC424_18_RECTYPE_AIRPORT_COMMUNICATIONS "PV"
#define ARINC424_18_RECTYPE_AIRPORT_TERMINALNDB "PN"
#define ARINC424_18_RECTYPE_AIRPORT_PATHPOINTSECTION "PP"
#define ARINC424_18_RECTYPE_AIRPORT_FLIGHTPLANNING "PR"
#define ARINC424_18_RECTYPE_AIRPORT_GNSS "PT"
#define ARINC424_18_RECTYPE_AIRPORT_TERMINALARRIVALALTITUDE "PK"

#define ARINC424_18_RECTYPE_COMPANYROUTE_SECTION "R "
#define ARINC424_18_RECTYPE_COMPANYROUTE_ALTERNATE "RA"

#define ARINC424_18_RECTYPE_AIRSPACE_RESTRICTIVE "UR"
#define ARINC424_18_RECTYPE_AIRSPACE_FIRUIR "UF"
#define ARINC424_18_RECTYPE_AIRSPACE_CONTROLLED "UC"

#define ARINC424_18_RECTYPE_CRUISINGTABLES_CRUISING "TC"
#define ARINC424_18_RECTYPE_CRUISINGTABLES_GEOREF "TG"

#define ARINC424_18_RECTYPE_MORA "AS"
#define ARINC424_18_RECTYPE_PREFERREDROUTES "ET"

#define ARINC424_18_RECTYPE_PREFERREDROUTES "ET"

#define ARINC424_18_RECTYPE_HELIPORT "HA"
#define ARINC424_18_RECTYPE_HELIPORT_TERMINALWAYPOINTS "HC"
#define ARINC424_18_RECTYPE_HELIPORT_SID "HD"
#define ARINC424_18_RECTYPE_HELIPORT_STAR "HE"
#define ARINC424_18_RECTYPE_HELIPORT_APPROACHES "HF"
#define ARINC424_18_RECTYPE_HELIPORT_MSA "HS"
#define ARINC424_18_RECTYPE_HELIPORT_COMMUNICATIONS "HV"
#define ARINC424_18_RECTYPE_HELIPORT_TERMINALARRIVALALTITUDE "HK"
/* End of Record Types */

/* Record structures.  Each field is an unterminated string */

/* Structures common to all records */
struct arinc424_18_footer_struct {
	char filerecord[5];
	char cycledate[4];
};

/* Structures common to both VHF and NDB records */
struct arinc424_18_navaid_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char icaoid[4];
	char icaocode[2];
	char blank1;
	char id[4];
	char blank2[2];
	char icao2[2];
	char continuation;
};

/* VHF Navaids Section code D aka SECCODE_NAVAID_VHF */
/* ARINC 424_18 section 4.1.2.1 */
struct arinc424_18_vhfnavaid_struct {
	struct arinc424_18_navaid_header_struct header;
	char frequency[5];
	char navaidclass[5];
	char lat[9];
	char lon[10];
	char dmeident[4];
	char dmelat[9];
	char dmelon[10];
	char declination[5];	/* East/West flag then 4 chars of value*10 */
	char dmeelevation[5];
	char figureofmerit;
	char ilsdmebias[2];
	char freqprotection[3];
	char datumcode[3];
	char name[30];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.2.2 */
struct arinc424_18_navaid_continuation_struct {
	struct arinc424_18_navaid_header_struct header;
	char application;
	char notes[69];
	char reserved[31];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.2.3 */
struct arinc424_18_vhfnavaid_simulation_continuation_struct {
	struct arinc424_18_navaid_header_struct header;
	char application;
	char blank[4];
	char facilitycharacteristics[5];
	char reserved1[42];
	char magneticvariation[5];
	char elevation[5];
	char reserved2[39];
	struct arinc424_18_footer_struct footer;
};

/* There is a VHF Navaid Flight Planning Continuation Update Record that is identical to
 * the arinc424_18_vfrnavaid_struct
 */
/* ARINC 424_18 section 4.1.2.4 */
#define arinc424_18_vhfnavaid_flightplanning_continuation_update_struct arinc424_18_vhfnavaid_struct

/* ARINC 424_18 section 4.1.2.5 */
struct arinc424_18_navaid_flightplanning_continuation_struct {
	struct arinc424_18_navaid_header_struct header;
	char application;
	char firid[4];
	char uirid[4];
	char startendindicator;
	char startenddate[11];
	char reserved[80];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.2.6 */
struct arinc424_18_vhfnavaid_limitation_continuation_struct {
	struct arinc424_18_navaid_header_struct header;
	char application;
	char limitation;
	char componentaffected;
	char sequence[2];
	struct {
		char sectorfromto[2];
		char distancedesc;
		char distancelimit[6];
		char altitudedesc;
		char alititudelimit[6];
	} limitations[5];
	char sequenceend;
	char blank[15];
	struct arinc424_18_footer_struct footer;
};

/* NDB Navaids Section code DB (or PN) aka SECCODE_NAVAID_NDB */
/* ARINC 424_18 section 4.1.3.1 */
struct arinc424_18_ndbnavaid_struct {
	struct arinc424_18_navaid_header_struct header;
	char frequency[5];
	char navaidclass[5];
	char lat[9];
	char lon[10];
	char blank1[23];
	char magvar[5];
	char blank2[6];
	char reserved[5];
	char datumcode[3];
	char name[30];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.3.2 */
struct arinc424_18_ndbnavaid_continuation_struct {
	struct arinc424_18_navaid_header_struct header;
	char application;
	char notes[69];
	char reserved[31];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.3.3 */
struct arinc424_18_ndbnavaid_simulation_continuation_struct {
	struct arinc424_18_navaid_header_struct header;
	char application;
	char blank[4];
	char facilitycharacteristics[5];
	char reserved1[47];
	char elevation[5];
	char reserved2[39];
	struct arinc424_18_footer_struct footer;
};

/* There is a NDB Navaid Flight Planning Continuation Update Record that is identical to
 * the arinc424_18_ndbnavaid_struct
 */
/* ARINC 424_18 section 4.1.3.4 */
#define arinc424_18_ndbnavaid_flightplanning_continuation_update_struct arinc424_18_ndbnavaid_struct

/* ARINC 424_18 section 4.1.3.5 is identical to 4.1.3.1 */

/* Waypoint Section code EA (or PC) aka SECCODE_ENROUTE_WAYPOINT */
struct arinc424_18_waypoint_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection_enroute;	/* Blank for ports */
	char region[4];				/* Enroute=ENRT Terminal=airport ICAO ID code */
	char icao[2];
	char subsection_port;		/* Blank for enroute */
	char id[5];
	char blank;
	char icao2[2];
	char continuation;
};

/* ARINC 424_18 section 4.1.4.1 */
struct arinc424_18_waypoint_struct {
	struct arinc424_18_waypoint_header_struct header;
	char blank1[4];
	char type[3];	/* Waypoint type */
	char usage[2];
	char blank2;
	char lat[9];
	char lon[10];
	char blank3[23];
	char dynamicmagvar[5];
	char reserved1[5];
	char datumcode[3];
	char reserved2[8];
	char nameformat[3];
	char name[25];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.4.2 */
struct arinc424_18_waypoint_continuation_struct {
	struct arinc424_18_waypoint_header_struct header;
	char application;
	char notes[69];
	char reserved[31];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.4.3 */
struct arinc424_18_waypoint_flightplanning_continuation_struct {
	struct arinc424_18_waypoint_header_struct header;
	char application;
	char firid[4];
	char uirid[4];
	char startendindicator;
	char startenddate[11];
	char reserved[80];
	struct arinc424_18_footer_struct footer;
};

/* There is a Waypoint Flight Planning Continuation Update Record that is identical to
 * the arinc424_18_waypoint_struct
 */
/* ARINC 424_18 section 4.1.4.4 */
#define arinc424_18_waypoint_flightplanning_continuation_update_struct arinc424_waypoint_struct

/* Holding Pattern code EP aka SECCODE_ENROUTE_HOLDINGPATTERNS */
struct arinc424_18_holdingpattern_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char region[4];				/* Enroute=ENRT Terminal=airport ICAO ID code */
	char icao[2];
	char blank[15];
	char dupid[2];
	struct {
		char id[5];
		char icao[2];
		char section;
		char subsection;
	} fix;
	char continuation;	
};

/* ARINC 424_18 section 4.1.5.1 */
struct arinc424_18_holdingpattern_struct {
	struct arinc424_18_holdingpattern_header_struct header;
	char inboundcourse[4];	/* Magnetic, tenths of a degree */
	char turndirection;
	char leglength[3];
	char legtime[2];
	char minaltitude[5];
	char maxaltitude[5];
	char speed[3];
	char rnp[3];
	char arcradius[6];
	char reserved[27];
	char name[25];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.5.2 */
struct arinc424_18_holdingpattern_continuation_struct {
	struct arinc424_18_holdingpattern_header_struct header;
	char application;
	char notes[69];
	char reserved[14];
	struct arinc424_18_footer_struct footer;
};

/* Enroute Airways code ER aka SECCODE_ENROUTE_AIRWAYS */
struct arinc424_18_airways_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char blank1[7];
	/* Some applications use a 6 character routeid and omit the reserved char */
	union {
		struct {
			char routeid[5];
			char reserved;
		};
		struct {
			char extrouteid[6];
		};
	};
	char blank2[6];
	char sequence[4];
	struct {
		char id[5];
		char icao[2];
		char section;
		char subsection;
	} fix;
	char continuation;	
};

/* ARINC 424_18 section 4.1.6.1 */
struct arinc424_18_airways_struct {
	struct arinc424_18_airways_header_struct header;
	char waypointdesc[4];
	char boundary;
	char routetype;
	char level;
	char directionrestriction;
	char cruisetable[2];
	char euindicator;
	char recommenednavaid[4];
	char icao[2];
	char rnp[3];
	char blank1[3];
	char theta[4];
	char rho[4];
	char outboundcourse[4];	/* Magnetic, tenths of a degree */
	char distancefrom[4];	/* Tenths of a nautical mile */
	char inboundcourse[4];	/* Magnetic, tenths of a degree */
	char blank2;
	char minaltitude1[5];
	char minaltitude2[5];
	char maxaltitude[5];
	char fixradiustransition[3];
	char reserved[22];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.6.2 */
struct arinc424_18_airways_continuation_struct {
	struct arinc424_18_airways_header_struct header;
	char application;
	char notes[69];
	char reserved[14];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.6.3 */
struct arinc424_18_airways_flightplanning_continuation_struct {
	struct arinc424_18_airways_header_struct header;
	char application;
	char startendindicator;
	char startenddate[11];
	char blank[14];
	struct {
		char icao[2];
		char type;
		char designation[10];
		char multiple;
	} restrictedairspace[4];
	char restrictedairspacelinkcont;
	struct arinc424_18_footer_struct footer;
};

/* There is an Airways Flight Planning Continuation Update Record that is identical to
 * the arinc424_18_airways_struct
 */
/* ARINC 424_18 section 4.1.6.4 */
#define arinc424_18_airways_flightplanning_continuation_update_struct arinc424_airways_struct

/* Airport code PA aka SECCODE_AIRPORT */
struct arinc424_18_airport_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank1;
	char icaoid[4];
	char icao[2];
	char subsection;
	char ataiata[3];
	char reserved[2];
	char blank2[3];
	char continuation;	
};

/* ARINC 424_18 section 4.1.7.1 */
struct arinc424_18_airport_struct {
	struct arinc424_18_airport_header_struct header;
	char speedlimitaltitude[5];
	char longestrunway[3];
	char ifr;
	char longestrunwaysurface;
	char lat[9];
	char lon[10];
	char magvar[5];
	char elevation[5];
	char speedlimit[3];
	char recommendednavaid[4];
	char icao[2];
	char transitionsaltitude[5];
	char transitionslevel[5];
	char publicmilitary;
	char timezone[3];
	char daylight;
	char magnetictrue;
	char datumcode[3];
	char reserved[4];
	char name[30];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.7.2 */
struct arinc424_18_airport_continuation_struct {
	struct arinc424_18_airport_header_struct header;
	char application;
	char notes[69];
	char reserved[31];	/* The spec inconsistently states that this length is 59 */
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.7.3 */
struct arinc424_18_airport_flightplanning_continuation_struct {
	struct arinc424_18_airport_header_struct header;
	char application;
	char fir[4];
	char uir[4];
	char startendindicator;
	char startenddate[11];
	char blank1[23];
	char controlledas;
	char controlledasairportid[4];
	char controlledasairporticao[2];
	char blank2[50];
	struct arinc424_18_footer_struct footer;
};

/* There is an Airport Flight Planning Continuation Update Record that is identical to
 * the arinc424_18_airport_struct
 */
/* ARINC 424_18 section 4.1.7.4 */
#define arinc424_18_airport_flightplanning_continuation_update_struct arinc424_airport_struct

/* Gate code PB aka SECCODE_AIRPORT_GATES */
struct arinc424_18_gates_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank1;
	char icaoid[4];
	char icao[2];
	char subsection;
	char gateid[5];
	char blank2[3];
	char continuation;	
};

/* ARINC 424_18 section 4.1.8.1 */
struct arinc424_18_gates_struct {
	struct arinc424_18_gates_header_struct header;
	char blank[10];
	char lat[9];
	char lon[10];
	char reserved[47];
	char name[25];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.8.2 */
struct arinc424_18_gates_continuation_struct {
	struct arinc424_18_gates_header_struct header;
	char application;
	char notes[69];
	char reserved[31];	/* The spec inconsistently states that this length is 59 */
	struct arinc424_18_footer_struct footer;
};

/* This structure is used for the following 3 types of records
 * SID code PB aka SECCODE_AIRPORT_SID
 * STAR code PE aka SECCODE_AIRPORT_STAR
 * Approach code PF aka SECCODE_AIRPORT_APPROACHES
 */
struct arinc424_18_sidstarapp_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank1;
	char id[4];
	char code[2];
	char subsection;
	char procedureid[6];
	char routetype;
	char transitionid[5];
	char blank2;
	char sequence[3];
	struct {
		char id[5];
		char icao[2];
		char section;
		char subsection;
	} fix;
	char continuation;
};

/* ARINC 424_18 section 4.1.9.1 */
struct arinc424_18_sidstarapp_struct {
	struct arinc424_18_sidstarapp_header_struct header;
	char waypointdesc[4];
	char turndir;
	char rnp[3];
	char pathandtermination[2];
	char turndirvalid;
	char recommendednavaid[4];
	char icao[2];
	char arcradius[6];
	char theta[4];
	char rho[4];
	char magcourse[4];
	char routedistholddisttime[4];
	char recdnavsection;
	char recdnavsubsection;
	char reserved[2];
	char altitudedesc;
	char atcindicator;
	char altitude[2][5];
	char transitionaltitude[5];
	char speedlimit[3];
	char verticalangle[4];
	char centerfixortaaprocturnindicator[5];
	char multiplecodetaasectorid;
	char icao2[2];
	char section2;
	char subsection2;
	char gnssfmsindication;
	char speedlimitdesc;
	char approutequalifier1;
	char approutequalifier2;
	char blank[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.9.2 */
struct arinc424_18_sidstarapp_primaryext_continuation_struct {
	struct arinc424_18_sidstarapp_header_struct header;
	char application;
	char catadh[4];
	char catbdh[4];
	char catcdh[4];
	char catddh[4];
	char catamda[4];
	char catbmda[4];
	char catcmda[4];
	char catdmda[4];
	char proceduretch[3];
	char localizeronlyaltitudedesc;
	char localizeronlyaltitude[5];
	char localizeronlyverticalangle[4];
	char blank1[4];
	char rnp[3];
	char reserved[26];
	char apchroutequalifier1;
	char apchroutequalifier2;
	char blank2[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.9.3 */
struct arinc424_18_sidstarapp_flightplanning_continuation_struct {
	struct arinc424_18_sidstarapp_header_struct header;
	char application;
	char startend;
	char startenddate[11];
	char blank[22];
	char legdist[4];
	char reserved[45];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.9.4 */
struct arinc424_18_sidstar_flightplanning_continuation_struct {
	struct arinc424_18_sidstarapp_header_struct header;
	char application;
	char startend;
	char startenddate[11];
	char blank[22];
	char legdist[4];
	char reserved[45];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.9.5 */
struct arinc424_18_procedure_continuation_struct {
	struct arinc424_18_sidstarapp_header_struct header;
	char application;
	char fasblockprovided;
	char fasblockprovidedlevelorservicename[10];
	char lnavvnavauthorizedforsbas;
	char lnavvnavlevelofservicename[10];
	char lnavauthorizedforsbas;
	char lnavlevelofservicename[10];
	char blank[45];
	char approutetypequalifier1;
	char approutetypequalifier2;
	char blank2[3];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code PG aka SECCODE_AIRPORT_RUNWAY
 */
struct arinc424_18_runway_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank1;
	char airportid[4];
	char icao[2];
	char subsection;
	char id[5];
	char blank2[3];
	char continuation;
};

/* ARINC 424_18 section 4.1.10.1 */
struct arinc424_18_runway_struct {
	struct arinc424_18_runway_header_struct header;
	char length[5];
	char magbearing[4];
	char blank3;
	char lat[9];
	char lon[10];
	char gradient[5];
	char blank4[4];
	char ellipsoidheight[6];
	char thresholdelevation[5];
	char displacedthresholddist[4];
	char thresholdcrossingheight[2];
	char width[3];
	char tch;
	char localizermlsglsrefpathid[4];
	char localizermlsglscategoryclass;
	char stopway[4];
	char localizermlsglsrefpathid2[4];
	char localizermlsglscategoryclass2;
	char reserved[6];
	char description[22];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.102. */
struct arinc424_18_runway_continuation_struct {
	struct arinc424_18_runway_header_struct header;
	char application;
	char notes[69];
	char reserved[31];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.10.3 */
struct arinc424_18_runway_simulation_continuation_struct {
	struct arinc424_18_runway_header_struct header;
	char application;
	char reserved[28];
	char truebearing[5];
	char truebearingsource;
	char reserved2[8];
	char tdzelocation;
	char tdzelevation[5];
	char reserved3[52];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code PI aka SECCODE_AIRPORT_LOCALIZERGLIDESLOPE
 */
struct arinc424_18_localizerglideslope_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank1;
	char airportid[4];
	char icao[2];
	char subsection;
	char id[4];
	char ilscategory;
	char blank2[3];
	char continuation;
};

/* ARINC 424_18 section 4.1.11.1 */
struct arinc424_18_localizerglideslope_struct {
	struct arinc424_18_localizerglideslope_header_struct header;
	char freq[5];
	char runwayid[5];
	char loclat[9];
	char loclon[10];
	char locbearing[4];
	char gslat[9];
	char gslon[10];
	char locpos[4];
	char locposref;
	char gspos[4];
	char locwidth[4];
	char gsangle[3];
	char declination[5];	/* East/West flag then 4 chars of value*10 */
	char gsheightatlandingthreshold[2];
	char gselevation[5];
	char supportingfacilityid[4];
	char supportingfacilityicao[2];
	char supportingfacilitysectioncode;
	char supportingfacilitysubsectioncode;
	char reserved[13];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.11.2 */
struct arinc424_18_localizerglideslope_continuation_struct {
	struct arinc424_18_localizerglideslope_header_struct header;
	char application;
	char notes[69];
	char reserved[31];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.11.3 */
struct arinc424_18_localizerglideslope_simulation_continuation_struct {
	struct arinc424_18_localizerglideslope_header_struct header;
	char application;
	char blank1[4];
	char facilitycharacteristics[5];
	char blank2[19];
	char loctruebearing[5];
	char locbearingsource;
	char reserved[30];
	char gsbeamwidth[3];
	char approachrouteident[5][6];
	char blank[3];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code R aka SECCODE_COMPANYROUTE_SECTION
 */
/* ARINC 424_18 section 4.1.12.1 */
struct arinc424_18_companyroute_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	struct {
		char id[5];
		char blank;
		char icao[2];
		char section;
		char subsection;
	} fromairportfix, toairportfix;
	char routeid[10];
	char sequence[3];
	char via[3];
	char sidstarappairway[6];
	char areacode[3];
	struct {
		char id[6];
		char icao[2];
		char section;
		char subsection;
	} tofix;
	char runwaytrans[5];
	char enrttrans[5];
	char reserved1;
	char cruisealt[5];
	char terminalalternateairport[4];
	char icao5[2];
	char alternatedistance[4];
	char costindex[3];
	char enroutealternateairport[4];
	char reserved2[29];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code PM aka SECCODE_AIRPORT_MARKERLOCALIZER
 */
struct arinc424_18_markerlocalizer_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	char airportid[4];
	char icao[2];
	char subsection;
	char localizerid[4];
	char markertype[3];
	char blank2;
	char continuation;
};

/* ARINC 424_18 section 4.1.13.1 */
struct arinc424_18_markerlocalizer_struct {
	struct arinc424_18_markerlocalizer_header_struct header;
	char locfreq[5];
	char runwayid[5];
	char markerlat[9];
	char markerlon[10];
	char minoraxisbearing[4];
	char loclat[9];
	char loclon[10];
	char locclass[5];
	char locfacilitycharacteristics[5];
	char locatorid[4];
	char blank[2];
	char magvar[5];
	char blank2[2];
	char elevation[5];
	char reserved[21];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.13.2 */
struct arinc424_18_markerlocalizer_continuation_struct {
	struct arinc424_18_markerlocalizer_header_struct header;
	char apptype;
	char reserved[100];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code PV aka SECCODE_AIRPORT_COMMUNICATIONS
 */
struct arinc424_18_comm_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	char airportid[4];
	char icao[2];
	char subsection;
	char commtype[3];
	char freq[7];
	char guardtransmit;
	char frequnits;
	char continuation;
};

/* ARINC 424_18 section 4.1.14.1 */
struct arinc424_18_comm_struct {
	struct arinc424_18_comm_header_struct header;
	char serviceindicator[3];
	char radarservice;
	char modulation;
	char signalemission;
	char lat[9];
	char lon[10];
	char magvar[5];
	char elevation[5];
	char h24indicator;
	char sectorization[6];
	char altitudedesc;
	char commalt[2][5];
	struct {
		char id[4];
		char icao[2];
		char section;
		char subsection;
	} sectorfacility;
	char distancedesc;
	char commdesc[2];
	struct {
		char id[4];
		char icao[2];
		char section;
		char subsection;
	} remotefacility;
	char callsign[25];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.14.2 */
struct arinc424_18_comm_continuation_struct {
	struct arinc424_18_comm_header_struct header;
	char applicationtype;
	char narrative[60];
	char reserved[36];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.14.3 */
struct arinc424_18_comm_additional_continuation_struct {
	struct arinc424_18_comm_header_struct header;
	char applicationtype;
	char timecode;
	char notam;
	char timeindicator;
	char timeofoperation[7][10];
	char reserved[23];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code EM aka SECCODE_ENROUTE_AIRWAYMARKER
 */
struct arinc424_18_airwaymarker_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char blank[7];
	char markerid[4];
	char blank2[2];
	char icao[2];
	char continuation;
};

/* ARINC 424_18 section 4.1.15.1 */
struct arinc424_18_airwaymarker_struct {
	struct arinc424_18_airwaymarker_header_struct header;
	char markercode[4];
	char reserved;
	char shape;
	char power;
	char blank[3];
	char lat[9];
	char lon[10];
	char minoraxis[4];
	char blank2[19];
	char magvar[5];
	char elevation[5];
	char datum[3];
	char blank3[6];
	char name[30];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.15.2 */
struct arinc424_18_airwaymarker_continuation_struct {
	struct arinc424_18_airwaymarker_header_struct header;
	char applicationtype;
	char reserved[100];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code TC aka SECCODE_CRUISINGTABLES_CRUISING
 */
/* ARINC 424_18 section 4.1.16.1 */
struct arinc424_18_cruisingtable_struct {
	char type;
	char blank[3];
	char section;
	char subsection;
	char tableid[2];
	char seq;
	char blank2[19];
	char coursefrom[4];
	char courseto[4];
	char magtrue;
	char blank3[2];
	struct {
		char from[5];
		char separation[5];
		char to[5];
	} level[4];
	char reserved[24];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code UF aka SECCODE_AIRSPACE_FIRUIR
 * FIR=Flight Information Region
 * UIR=Upper Flight Information Region
 */
struct arinc424_18_firuir_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char id[4];
	char address[4];
	char indicator;
	char sequence[4];
	char continuation;
};

/* ARINC 424_18 section 4.1.17.1 */
struct arinc424_18_firuir_struct {
	struct arinc424_18_firuir_header_struct header;
	char adjacentfir[4];
	char adjacentuir[4];
	char reportingunitsspeed;
	char reportingunitsaltitude;
	char entryreport;
	char blank;
	char boundaryvia[2];
	char lat[9];
	char lon[10];
	struct {
		char originlat[9];
		char originlon[10];
		char distance[4];
		char bearing[4];
	} arc;
	char firupperlimit[5];
	char uirlowerlimit[5];
	char uirupperlimit[5];
	char cruisetableind[2];
	char reserved;
	char name[25];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.17.2 */
struct arinc424_18_firuir_continuation_struct {
	struct arinc424_18_firuir_header_struct header;
	char application;
	char reserved[102];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code UR aka SECCODE_AIRSPACE_RESTRICTIVE
 */
struct arinc424_18_restrictive_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char icao[2];
	char restrictivetype;
	char designation[10];
	char multiple;
	char sequence[4];
	char continuation;
};

/* ARINC 424_18 section 4.1.18.1 */
struct arinc424_18_restrictive_struct {
	struct arinc424_18_restrictive_header_struct header;
	char level;
	char timecode;
	char notam;
	char blank[2];
	char boundaryvia[2];
	char lat[9];
	char lon[10];
	struct {
		char originlat[9];
		char originlon[10];
		char distance[4];
		char bearing[4];
	} arc;
	char blank2[3];
	struct {
		char limit[5];
		char unitindicator;
	} lower,upper;
	char name[30];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.18.2 */
struct arinc424_18_restrictive_continuation_struct {
	struct arinc424_18_restrictive_header_struct header;
	char application;
	char timecode;
	char notam;
	char timeindicator;
	char timeofoperations[7][10];
	char controllingagency[24];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.18.3 */
struct arinc424_18_restrictive_flightplanning_continuation_struct {
	struct arinc424_18_restrictive_header_struct header;
	char application;
	char blank[3];
	char startendindicator;
	char startenddate[11];
	char reserved[82];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code AS aka SECCODE_MORA
 */
/* ARINC 424_18 section 4.1.19.1 */
struct arinc424_18_mora_struct {
	char type;
	char blank[3];
	char section;
	char subsection;
	char blank2[7];
	char lat[3];
	char lon[4];
	char blank3[10];
	char mora[30][3];
	char reserved[3];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code PS aka SECCODE_AIRPORT_MSA
 */
struct arinc424_18_msa_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	char id[4];
	char icao[2];
	char subsection;
	struct {
		char id[5];
		char icao[2];
		char section;
		char subsection;
	} center;
	char multiple;
	char reserved[15];
	char continuation;
};

/* ARINC 424_18 section 4.1.20.1 */
struct arinc424_18_msa_struct {
	struct arinc424_18_msa_header_struct header;
	char reserved[3];
	char magnetictrue;
	struct {
		char bearing[6];
		char altitude[3];
		char radius[2];
	} sector[7];
	char reserved2[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.20.2 */
struct arinc424_18_msa_continuation_struct {
	struct arinc424_18_msa_header_struct header;
	char application;
	char notes[69];
	char reserved[14];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code EU aka SECCODE_ENROUTE_AIRWAYSRESTRICTIONS
 */
struct arinc424_18_airwaysrestrictions_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	/* Some applications use a 6 character routeid and omit the reserved char */
	union {
		struct {
			char routeid[5];
			char reserved;
		};
		struct {
			char extrouteid[6];
		};
	};
	char id[3];
	char restrictiontype[2];
	char continuation;
};

/* ARINC 424_18 section 4.1.21.1 */
struct arinc424_18_airwaysrestrictions_altitude_struct {
	struct arinc424_18_airwaysrestrictions_header_struct header;
	char startfixid[5];
	char startfixicao[2];
	char startfixsection;
	char startfixsubsection;
	char endfixid[5];
	char endfixicao[2];
	char endfixsection;
	char endfixsubsection;
	char blank;
	char startdate[7];
	char enddate[7];
	char timecode;
	char timeindicator;
	char timeofoperation[4][10];
	char exclusionindicator;
	char unitsofaltitude;
	struct {
		char altitude[3];
		char blockindicator;
	} restriction[7];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.21A.1 */
struct arinc424_18_airwaysrestrictions_note_struct {
	struct arinc424_18_airwaysrestrictions_header_struct header;
	char startfixid[5];
	char startfixicao[2];
	char startfixsection;
	char startfixsubsection;
	char endfixid[5];
	char endfixicao[2];
	char endfixsection;
	char endfixsubsection;
	char blank;
	char startdate[7];
	char enddate[7];
	char notes[69];
	char blank2[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.21A.2 */
struct arinc424_18_airwaysrestrictions_note_continuation_struct {
	struct arinc424_18_airwaysrestrictions_header_struct header;
	char application;
	char reserved[32];
	char notes[69];
	char blank[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.21B.1 */
struct arinc424_18_airwaysrestrictions_seasonalclosure_struct {
	struct arinc424_18_airwaysrestrictions_header_struct header;
	char startfixid[5];
	char startfixicao[2];
	char startfixsection;
	char startfixsubsection;
	char endfixid[5];
	char endfixicao[2];
	char endfixsection;
	char endfixsubsection;
	char blank;
	char startdate[7];
	char enddate[7];
	char timecode;
	char timeindicator;
	char timeofoperation[4][10];
	char cruisetable[2];
	char blank2[28];
	struct arinc424_18_footer_struct footer;
};

/* These structures are identical */
/* ARINC 424_18 section 4.1.21C.1 */
#define arinc424_18_airwaysrestrictions_cruisingtablereplacement_struct arinc424_18_airwaysrestrictions_seasonalclosure_struct

/* ARINC 424_18 section 4.1.21C.2 */
struct arinc424_18_airwaysrestrictions_cruisingtablereplacement_continuation_struct {
	struct arinc424_18_airwaysrestrictions_header_struct header;
	char application;
	char reserved[32];
	char timecode;
	char timeindicator;
	char timeofoperation[4][10];
	char cruisetable[2];
	char blank[28];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.21.2 */
struct arinc424_18_airwaysrestrictions_altitude_continuation_struct {
	struct arinc424_18_airwaysrestrictions_header_struct header;
	char application;
	char reserved[32];
	char timecode;
	char timeindicator;
	char timeofoperation[4][10];
	char exclusionindicator;
	char unitsofaltitude;
	struct {
		char altitude[3];
		char blockindicator;
	} restriction[7];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code PL aka SECCODE_AIRPORT_MLS
 */
struct arinc424_18_mls_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	char airportid[4];
	char icao[2];
	char subsection;
	char mlsid[4];
	char category;
	char blank2[3];
	char continuation;
};

/* ARINC 424_18 section 4.1.22.1 */
struct arinc424_18_mls_struct {
	struct arinc424_18_mls_header_struct header;
	char channel[3];
	char blank[2];
	char runwayid[5];
	char azimuthlat[9];
	char azimuthlon[10];
	char azimuthbearing[4];
	char elevationlat[9];
	char elevationlon[10];
	char azimuthpos[4];
	char azimuthposref;
	char elevationpos[4];
	char azimuthproportionalangleright[3];
	char azimuthproportionalangleleft[3];
	char azimuthcoverageleft[3];
	char azimuthcoverageright[3];
	char elevationanglespan[3];
	char magvar[5];
	char elelevation[5];
	char nominalelevationangle[4];
	char minimumglidepathangle[3];
	char supportingfacilityid[4];
	char supportingfacilityicao[2];
	char supportingfacilitysection;
	char supportingfacilitysubsection;
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.22.2 */
struct arinc424_18_mls_continuation_struct {
	struct arinc424_18_mls_header_struct header;
	char application;
	char blank[4];
	char facilitycharacteristics[5];
	char backazimuthlat[9];
	char backazimuthlon[10];
	char backazimuthbearing[4];
	char datumpointlat[9];
	char datumpointlon[10];
	char backazimuthpos[4];
	char backazimuthposref;
	char blank2[4];
	char backazimuthproportionalangleright[3];
	char backazimuthproportionalangleleft[3];
	char backazimuthcoverageleft[3];
	char backazimuthcoverageright[3];
	char backazimuthtruebearing[5];
	char backazimuthtruebearingsource;
	char azimuthtruebearing[5];
	char azimuthbearingsource;
	char glidepathheightatthreshold[2];
	char reserved[14];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code EV aka SECCODE_ENROUTE_COMMUNICATIONS
 */
struct arinc424_18_enroutecomm_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char firrdoid[4];
	char firuiraddress[4];
	char indicator;
	char reserved[3];
	char remotename[25];
	char commtype[3];
	char freq[7];
	char guardtransmit;
	char units;
	char continuation;
};

/* ARINC 424_18 section 4.1.23.1 */
struct arinc424_18_enroutecomm_struct {
	struct arinc424_18_enroutecomm_header_struct header;
	char serviceindicator[3];
	char radarservice;
	char modulation;
	char signalemission;
	char lat[9];
	char lon[10];
	char magvar[5];
	char elevation[5];
	char h24indicator;
	char altitudedesc;
	char commaltitude[2][5];
	struct {
		char facility[4];
		char icao[2];
		char section;
		char subsection;
	} remote;
	char reserved[12];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.23.2 */
struct arinc424_18_enroutecomm_continuation_struct {
	struct arinc424_18_enroutecomm_header_struct header;
	char application;
	char timecode;
	char notam;
	char timeindicator;
	char timeofoperation[10];
	char reserved[23];
	char callsign[30];
	struct arinc424_18_footer_struct footer;
};

/* This structure and the one above are titled exactly the same way
 * in the spec - not sure what the distinction is
 */
/* ARINC 424_18 section 4.1.23.3 */
struct arinc424_18_enroutecomm_continuation2_struct {
	struct arinc424_18_enroutecomm_header_struct header;
	char application;
	char blank[3];
	char timeofoperation[6][10];
	char reserved[3];
	struct arinc424_18_footer_struct footer;
};

/*
 * Preferred Route ET aka SECCODE_PREFERREDROUTES
 */
struct arinc424_18_preferredroute_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char blank[7];
	char routeid[10];
	char useind[2];
	char sequence[4];
	char blank2[9];
	char continuation;
};

/* ARINC 424_18 section 4.1.24.1 */
struct arinc424_18_preferredroute_struct {
	struct arinc424_18_preferredroute_header_struct header;
	struct {
		char id[5];
		char icao[2];
		char section;
		char subsection;
	} tofix;
	char via[3];
	char sidstarairway[6];
	char area[3];
	char level;
	char type;
	struct {
		char airportfix[5];
		char icao[2];
		char section;
		char subsection;
	} initial,terminus;
	char minalt[5];
	char maxalt[5];
	char timecode;
	char aircraftusegroup[2];
	char directionrestriction;
	char altitudedescription;
	char altitude[2][5];
	char reserved[18];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.24.2 */
struct arinc424_18_preferredroute_continuation_struct {
	struct arinc424_18_preferredroute_header_struct header;
	char application;
	char timecode;
	char timeindicator;
	char timeofoperation[7][10];
	char reserved[11];
	struct arinc424_18_footer_struct footer;
};

/* This structure and the one above are titled almost exactly the same way
 * in the spec - not sure what the distinction is
 */
/* ARINC 424_18 section 4.1.24.3 */
struct arinc424_18_preferredroute_continuation2_struct {
	struct arinc424_18_preferredroute_header_struct header;
	char application;
	char notes[69];
	char reserved[14];
	struct arinc424_18_footer_struct footer;
};

/*
 * Preferred Route UC aka SECCODE_AIRSPACE_CONTROLLED
 */
struct arinc424_18_airspace_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char icao[2];
	char airspacetype;
	struct {
		char id[5];
		char section;
		char subsection;
	} center;
	char classification;
	char reserved[2];
	char multiple;
	char sequence[4];
	char continuation;
};

/* ARINC 424_18 section 4.1.25.1 */
struct arinc424_18_airspace_struct {
	struct arinc424_18_airspace_header_struct header;
	char level;
	char timecode;
	char notam;
	char blank[2];
	char boundaryvia[2];
	char lat[9];
	char lon[10];
	struct {
		char originlat[9];
		char originlon[10];
		char distance[4];
		char bearing[4];
	} arc;
	char rnp[3];
	struct {
		char limit[5];
		char unitindicator;
	} lower,upper;
	char name[30];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.25.2 */
struct arinc424_18_airspace_continuation_struct {
	struct arinc424_18_airspace_header_struct header;
	char application;
	char timecode;
	char notam;
	char timeindicator;
	char timeofoperatoin[7][10];
	char controllingagency[24];
	struct arinc424_18_footer_struct footer;
};

/*
 * Geographical Reference Table TG aka SECCODE_CRUISINGTABLES_GEOREF
 */
struct arinc424_18_georef_header_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	char tableid[2];
	char sequence;
	char entity[29];
	char continuation;	
};

/* ARINC 424_18 section 4.1.26.1 */
struct arinc424_18_georef_struct {
	struct arinc424_18_georef_header_struct header;
	char reserved;
	struct {
		char id[10];
		char useindicator[2];
	} preferredroute[6];
	char blank[11];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.26.2 */
struct arinc424_18_georef_continuation_struct {
	struct arinc424_18_georef_header_struct header;
	char application;
	char reserved[83];
	struct arinc424_18_footer_struct footer;
};

/*
 * Flight Planning PR aka SECCODE_AIRPORT_FLIGHTPLANNING
 */
struct arinc424_18_flightplanning_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	char airportid[4];
	char icao[2];
	char subsection;
	char sidstarapproachid[6];
	char proceduretype;
	struct {
		char transitionid[5];
		char transitionfix[5];
		char icao[2];
		char section;
		char subsection;
		char transitionalongtrackdistance[3];
	} runway;
	struct {
		char fix[5];
		char icao[2];
		char section;
		char subsection;
		char segmentalongtrackdistance[3];
	} commonsegmenttransition;
	struct {
		char id[5];
		char fix[5];
		char icao[2];
		char section;
		char subsection;
		char transitionalongtrackdistance[3];
	} enroutetransition;
	char sequence[3];
	char continuation;
};

/* ARINC 424_18 section 4.1.27.1 */
struct arinc424_18_flightplanning_struct {
	struct arinc424_18_flightplanning_header_struct header;
	char engines[4];
	char turbopropjet;
	char rnav;
	char atcweightcategory;
	char atcid[7];
	char timecode;
	char proceduredescription[15];
	char legtype[2];
	char reportingcode;
	char initaldeparturemagneticcourse[4];
	char altitudedesc;
	char altitude[2][3];
	char speedlimit[3];
	char initialcruisetable[2];
	char speedlimitdescription;
	char blank[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.27.2 */
struct arinc424_18_flightplanning_continuation_struct {
	struct arinc424_18_flightplanning_header_struct header;
	char application;
	struct {
		char id[5];
		char icao[2];
		char section;
		char subsection;
		char distance[3];
		char fixrelatedtransitioncode;
	} intermediatefix[4];
	struct arinc424_18_footer_struct footer;
};

/* This structure and the one above are titled almost exactly the same way
 * in the spec - not sure what the distinction is
 */
/* ARINC 424_18 section 4.1.27.3 */
struct arinc424_18_flightplanning_continuation2_struct {
	struct arinc424_18_flightplanning_header_struct header;
	char application;
	char timecode;
	char timeindicator;
	char timeofoperation[5][10];
	struct arinc424_18_footer_struct footer;
};

/*
 * Path Point PP aka SECCODE_AIRPORT_PATHPOINTSECTION
 */
struct arinc424_18_pathpoint_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	struct {
		char id[4];
		char icao[2];
		char subsection;
	} airport;
	char approachprocedure[6];
	char runwayorhelipadid[5];
	char operationtype[2];
	char continuation;
};

/* ARINC 424_18 section 4.1.28.1 */
struct arinc424_18_pathpoint_struct {
	struct arinc424_18_pathpoint_header_struct header;
	char routeindicator;
	char sbasserviceprovider[2];
	char referencepathdataselector[2];
	char referencepathid[4];
	char approachperformancedesignator;
	char thresholdlandingpointlat[11];
	char thresoldlandingpointlon[12];
	char ltpellipsoidheight[6];
	char glidepathangle[4];
	char flightpathalignmentpointlat[11];
	char flightpathalignmentpointlon[12];
	char coursewidthatthreshold[5];
	char lengthoffset[4];
	char pathpointtch[6];
	char tchunitsindicator;
	char hal[3];
	char val[3];
	char sbasfasdatacrcremainder[8];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.28.2 */
struct arinc424_18_pathpoint_continuationstruct {
	struct arinc424_18_pathpoint_header_struct header;
	char application;
	char fpapellipsoidheight[6];
	char fpaporthometricheight[6];
	char ltporthometricheight[6];
	char approachtypeid[10];
	char gnsschannel[5];
	char blank[10];
	char helicopterprocedurecourse[3];
	char blank2[49];
	struct arinc424_18_footer_struct footer;
};

/*
 * GLS Record PT aka SECCODE_AIRPORT_GNSS
 */
struct arinc424_18_gls_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	struct {
		char id[4];
		char icao[2];
		char subsection;
	} airport;
	char refpathid[4];
	char category;
	char blank2[3];
	char continuation;
};

/* ARINC 424_18 section 4.1.29.1 */
struct arinc424_18_gls_struct {
	struct arinc424_18_gls_header_struct header;
	char channel[5];
	char runwayid[5];
	char blank[19];
	char approachbearing[4];
	struct {
		char lat[9];
		char lon[10];
		char id[4];
	} station;
	char blank2[5];
	char servicevolumeradius[2];
	char tdmaslots[2];
	char approachslope[3];
	char magvar[5];
	char reserved[2];
	char elevation[5];
	char datum[3];
	char stationtype[3];
	char blank3[2];
	char stationelevationwgs84[5];
	char blank4[8];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.1.29.2 */
struct arinc424_18_gls_continuation_struct {
	struct arinc424_18_gls_header_struct header;
	char application;
	char reserved[100];
	struct arinc424_18_footer_struct footer;
};

/*
 * Alternate Record RA aka SECCODE_COMPANYROUTE_ALTERNATE
 */
/* ARINC 424_18 section 4.1.30.1 */
struct arinc424_18_alternate_struct {
	char type;
	char customerarea[3];
	char section;
	char subsection;
	struct {
		char airportorfix[5];
		char icao[2];
		char section;
		char subsection;
		char type[2];
	} related;
	struct {
		char blank[2];
		char distance[3];
		char type;
		char id[10];
	} alternate[6];		/* The first one is the primary */
	char reserved[10];
	struct arinc424_18_footer_struct footer;
};

/*
 * Airport TAA (Terminal Arrival Altitude) PK aka SECCODE_AIRPORT_TERMINALARRIVALALTITUDE
 */
struct arinc424_18_taa_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	struct {
		char id[4];
		char icao[2];
		char subsection;
	} airport;
	char id[6];
	char sectorid;
	char procedureturn[4];
	char blank2[5];
	struct {
		char waypoint[5];
		char icao[2];
		char section;
		char subsection;
	} iaf;
	char continuation;
};

/* ARINC 424_18 section 4.1.31.1 */
struct arinc424_18_taa_struct {
	struct arinc424_18_taa_header_struct header;
	char reserved;
	char magtrueindicator;
	struct {
		char radius[4];
		char bearing[6];
		char minalt[3];
	} sector[6];
	char blank[4];
	struct arinc424_18_footer_struct footer;
};	

/* ARINC 424_18 section 4.1.31.2 */
struct arinc424_18_taa_continuation_struct {
	struct arinc424_18_taa_header_struct header;
	char application;
	char notes[69];
	char reserved[14];
	struct arinc424_18_footer_struct footer;
};

/*
 * Heliport Records HA aka 
 */
struct arinc424_18_heliport_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	struct {
		char id[4];
		char icao[2];
		char subsection;
	} heliport;
	char ataiata[3];
	char padid[5];
	char continuation;	
};

/* ARINC 424_18 section 4.2.1.1 */
struct arinc424_18_heliport_struct {
	struct arinc424_18_heliport_header_struct header;
	char speedlimitalt[5];
	char datumcode[3];
	char ifrindicator;
	char blank;
	char lat[9];
	char lon[10];
	char magvar[5];
	char elevation[5];
	char speedlimit[3];
	char recommendedvhfnavaid[4];
	char navaidicao[2];
	char transitionaltitude[5];
	char transitionlevel[5];
	char publicmilitaryindicator;
	char timezone[3];
	char daylightindicator;
	char paddimensions[6];
	char magnetictrue;
	char reserved;
	char name[30];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.1.2 */
struct arinc424_18_heliport_continuation_struct {
	struct arinc424_18_heliport_header_struct header;
	char application;
	char notes[69];
	char reserved[31];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.1.3 */
struct arinc424_18_heliport_flightplanning_continuation_struct {
	struct arinc424_18_heliport_header_struct header;
	char application;
	char firid[4];
	char uirid[4];
	char startendindicator;
	char startenddatetime[11];
	char blank[23];
	struct {
		char indicator;
		char airportid[4];
		char airporticao[2];
	} controlledas;
	char blank2[50];
	struct arinc424_18_footer_struct footer;
};

/* There is a Heliport Flight Planning Continuation Update Record that is identical to
 * the arinc424_18_heliport_continuation_struct
 */
/* ARINC 424_18 section 4.2.1.4 */
#define arinc424_18_heliport_flightplanning_continuation2_struct arinc424_18_heliport_continuation_struct

/*
 * Heliport Terminal Waypoint Record HC aka SECCODE_HELIPORT_TERMINALWAYPOINTS
 */
struct arinc424_18_heliport_terminalwaypoint_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	struct {
		char id[4];
		char icao[2];
		char subsection;
	} heliport;
	struct {
		char id[5];
		char blank;
		char icao[2];
	} waypoint;
	char continuation;	
};

/* ARINC 424_18 section 4.2.2.1 */
struct arinc424_18_heliport_terminalwaypoint_struct {
	struct arinc424_18_heliport_terminalwaypoint_header_struct header;
	char blank[4];
	char type[3];
	char usage[2];
	char blank2;
	char lat[9];
	char lon[10];
	char blank3[23];
	char dynmagvar[5];
	char reserved[5];
	char datum[3];
	char reserved2[8];
	char nameformat[3];
	char name[25];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.2.2 */
struct arinc424_18_heliport_terminalwaypoint_continuation_struct {
	struct arinc424_18_heliport_terminalwaypoint_header_struct header;
	char application;
	char notes[69];
	char reserved[31];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.2.3 */
struct arinc424_18_heliport_terminalwaypoint_flightplanning_continuation_struct {
	struct arinc424_18_heliport_header_struct header;
	char application;
	char firid[4];
	char uirid[4];
	char startendindicator;
	char startenddatetime[11];
	char reserved[80];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.2.4 is identical to 4.2.2.1 */

/* This structure is used for the following 3 types of records
 * SID code HD aka SECCODE_HELIPORT_SID
 * STAR code HE aka SECCODE_HELIPORT_STAR
 * Approach code HF aka SECCODE_HELIPORT_APPROACHES
 */
struct arinc424_18_heliport_sidstarapp_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank1;
	char icaoid[4];
	char icao[2];
	char subsection;
	char procedureid[6];
	char routetype;
	char transitionid[5];
	char blank2;
	char sequence[3];
	struct {
		char id[5];
		char icao[2];
		char section;
		char subsection;
	} fix;
	char continuation;
};

/* ARINC 424_18 section 4.2.3.1 */
struct arinc424_18_heliport_sidstarapp_struct {
	struct arinc424_18_heliport_sidstarapp_header_struct header;
	char waypointdesc[4];
	char turndir;
	char rnp[3];
	char pathandtermination[2];
	char turndirvalid;
	char recommendednavaid[4];
	char icao[2];
	char arcradius[6];
	char theta[4];
	char rho[4];
	char magcourse[4];
	char routedistholddisttime[4];
	char recdnavsection;
	char recdnavsubsection;
	char reserved[2];
	char altitudedesc;
	char atcindicator;
	char altitude[2][5];
	char transitionaltitude[5];
	char speedlimit[3];
	char verticalangle[4];
	char centerfixortaaprocturnindicator[5];
	char multiplecodetaasectorid;
	char icao2[2];
	char section2;
	char subsection2;
	char gnssfmsindication;
	char speedlimitdesc;
	char approutequalifier1;
	char approutequalifier2;
	char blank[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.3.2 */
struct arinc424_18_heliport_sidstarapp_primaryext_continuation_struct {
	struct arinc424_18_heliport_sidstarapp_header_struct header;
	char application;
	char catadh[4];
	char catbdh[4];
	char catcdh[4];
	char catddh[4];
	char catamda[4];
	char catbmda[4];
	char catcmda[4];
	char catdmda[4];
	char proceduretch[3];
	char localizeraltitudedesc;
	char localizeraltitude[5];
	char localizervnav[4];
	char blank1[4];
	char rnp[3];
	char reserved[26];
	char apchroutequalifier1;
	char apchroutequalifier2;
	char blank2[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.3.3 */
struct arinc424_18_heliport_sidstarapp_flightplanning_continuation_struct {
	struct arinc424_18_heliport_sidstarapp_header_struct header;
	char application;
	char startend;
	char startenddate[11];
	char blank[22];
	char routedist[4];
	char reserved[45];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.3.4 is identical to 4.2.3.1 */

/* ARINC 424_18 section 4.2.3.5 */
struct arinc424_18_heliport_procedure_continuation_struct {
	struct arinc424_18_heliport_sidstarapp_header_struct header;
	char application;
	char fasblockprovided;
	char fasblockprovidedlevelorservicename[10];
	char lnavvnavauthorizedforsbas;
	char lnavvnavlevelofservicename[10];
	char lnavauthorizedforsbas;
	char lnavlevelofservicename[10];
	char blank[45];
	char approutetypequalifier1;
	char approutetypequalifier2;
	char blank2[3];
	struct arinc424_18_footer_struct footer;
};

/*
 * Approach code HS aka SECCODE_HELIPORT_MSA
 */
struct arinc424_18_heliport_msa_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	char id[4];
	char icao[2];
	char subsection;
	struct {
		char id[5];
		char icao[2];
		char section;
		char subsection;
	} center;
	char multiple;
	char reserved[15];
	char continuation;
};

/* ARINC 424_18 section 4.2.4.1 */
struct arinc424_18_heliport_msa_struct {
	struct arinc424_18_heliport_msa_header_struct header;
	char reserved[3];
	struct {
		char bearing[6];
		char altitude[3];
		char radius[2];
	} sector[7];
	char magnetictrue;
	char reserved2[3];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.4.2 */
struct arinc424_18_heliport_msa_continuation_struct {
	struct arinc424_18_heliport_msa_header_struct header;
	char application;
	char notes[69];
	char reserved[14];
	struct arinc424_18_footer_struct footer;
};

/*
 * Heliport Communications HV aka SECCODE_HELIPORT_COMMUNICATIONS
 */
struct arinc424_18_heliport_comm_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	char id[4];
	char icao[2];
	char subsection;
	char commtype[3];
	char freq[7];
	char guardtransmit;
	char frequnits;
	char continuation;
};

/* ARINC 424_18 section 4.2.5.1 */
struct arinc424_18_heliport_comm_struct {
	struct arinc424_18_heliport_comm_header_struct header;
	char serviceindicator[3];
	char radarservice;
	char modulation;
	char signalemission;
	char lat[9];
	char lon[10];
	char magvar[5];
	char elevation[5];
	char h24indicator;
	char sectorization[6];
	char altitudedesc;
	char commalt[2][5];
	struct {
		char id[4];
		char icao[2];
		char section;
		char subsection;
	} sectorfacility;
	char distancedesc;
	char commdesc[2];
	struct {
		char id[4];
		char icao[2];
		char section;
		char subsection;
	} remotefacility;
	char callsign[25];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.5.2 */
struct arinc424_18_heliport_comm_continuation_struct {
	struct arinc424_18_heliport_comm_header_struct header;
	char applicationtype;
	char narrative[60];
	char reserved[36];
	struct arinc424_18_footer_struct footer;
};

/* ARINC 424_18 section 4.2.5.3 */
struct arinc424_18_heliport_comm_additional_continuation_struct {
	struct arinc424_18_heliport_comm_header_struct header;
	char applicationtype;
	char timecode;
	char notam;
	char timeindicator;
	char timeofoperation[7][10];
	char reserved[23];
	struct arinc424_18_footer_struct footer;
};

/*
 * Airport TAA (Terminal Arrival Altitude) HK aka SECCODE_HELIPORT_TERMINALARRIVALALTITUDE
 */
struct arinc424_18_heliport_taa_header_struct {
	char type;
	char customerarea[3];
	char section;
	char blank;
	struct {
		char id[4];
		char icao[2];
		char subsection;
	} heliport;
	char id[6];
	char sectorid;
	char procedureturn[4];
	char blank2[5];
	struct {
		char waypoint[5];
		char icao[2];
		char section;
		char subsection;
	} iaf;
	char continuation;
};

/* ARINC 424_18 section 4.2.6.1 */
struct arinc424_18_heliport_taa_struct {
	struct arinc424_18_heliport_taa_header_struct header;
	char reserved;
	char magtrueindicator;
	struct {
		char radius[4];
		char bearing[6];
		char minalt[3];
	} sector[6];
	char blank[4];
	struct arinc424_18_footer_struct footer;
};	

/* ARINC 424_18 section 4.2.6.2 */
struct arinc424_18_heliport_taa_continuation_struct {
	struct arinc424_18_heliport_taa_header_struct header;
	char application;
	char notes[69];
	char reserved[14];
	struct arinc424_18_footer_struct footer;
};

/* Section 5.36 Latitude */
/* Returns the latitude on success or NaN on error */
double arinc424_18_parse_lat(const char lat[9]);

/* Section 5.37 Longitude */
/* Returns the longitude on success or NaN on error */
double arinc424_18_parse_lon(const char lon[10]);

/* Return the Section and Subsection (Sections 5.4 and 5.5) from a record as a string */
char *arinc424_18_parse_recordtype(const char* line);

/* Return the unscaled frequency (Section 5.103) */
unsigned int arinc424_18_parse_freq(const char* freqfield);

#endif /*ARINC424_18_H_*/
