#ifndef AIRCRAFTDIALOG_H_
#define AIRCRAFTDIALOG_H_

#include <gtkmm/dialog.h>
#include <gtkmm/combobox.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/liststore.h>
#include <libglademm.h>
#include <string>
#include "Aircraft.h"

class AircraftDialog
{
public:
	AircraftDialog(std::string *GladeFilename, Aircraft *AircraftData);
	~AircraftDialog();
	int Run();
	void SetSensitivities();
	void GetData();
	void SetData();

protected:

	class AirspeedModelColumns : public Gtk::TreeModel::ColumnRecord {
	public:
		AirspeedModelColumns(Gtk::ComboBox *Combo);
	protected:
		Gtk::TreeModelColumn<int> Id;
		Gtk::TreeModelColumn<Glib::ustring> Name;
		Glib::RefPtr<Gtk::ListStore> AirspeedrefTreeModel;
	};
	AirspeedModelColumns *AirspeedComboColumns;

	class ClimbDescentRateModelColumns : public Gtk::TreeModel::ColumnRecord {
	public:
		ClimbDescentRateModelColumns(Gtk::ComboBox *Combo);
	protected:
		Gtk::TreeModelColumn<int> Id;
		Gtk::TreeModelColumn<Glib::ustring> Name;
		Glib::RefPtr<Gtk::ListStore> ClimbDescentRaterefTreeModel;
	};
	ClimbDescentRateModelColumns *ClimbDescentRateComboColumns;

	class AltitudeModelColumns : public Gtk::TreeModel::ColumnRecord {
	public:
		AltitudeModelColumns(Gtk::ComboBox *Combo);
	protected:
		Gtk::TreeModelColumn<int> Id;
		Gtk::TreeModelColumn<Glib::ustring> Name;
		Glib::RefPtr<Gtk::ListStore> AltituderefTreeModel;
	};
	AltitudeModelColumns *AltitudeComboColumns;

	class AircraftWeightModelColumns : public Gtk::TreeModel::ColumnRecord {
	public:
		AircraftWeightModelColumns(Gtk::ComboBox *Combo);
	protected:
		Gtk::TreeModelColumn<int> Id;
		Gtk::TreeModelColumn<Glib::ustring> Name;
		Glib::RefPtr<Gtk::ListStore> AircraftWeightrefTreeModel;
	};
	AircraftWeightModelColumns *AircraftWeightComboColumns;

	class CGStationDistModelColumns : public Gtk::TreeModel::ColumnRecord {
	public:
		CGStationDistModelColumns(Gtk::ComboBox *Combo);
	protected:
		Gtk::TreeModelColumn<int> Id;
		Gtk::TreeModelColumn<Glib::ustring> Name;
		Glib::RefPtr<Gtk::ListStore> CGStationDistrefTreeModel;
	};
	CGStationDistModelColumns *CGStationDistComboColumns;

	class OilQtyModelColumns : public Gtk::TreeModel::ColumnRecord {
	public:
		OilQtyModelColumns(Gtk::ComboBox *Combo);
	protected:
		Gtk::TreeModelColumn<int> Id;
		Gtk::TreeModelColumn<Glib::ustring> Name;
		Glib::RefPtr<Gtk::ListStore> OilQtyrefTreeModel;
	};
	OilQtyModelColumns *OilQtyComboColumns;

	class FuelQtyModelColumns : public Gtk::TreeModel::ColumnRecord {
	public:
		FuelQtyModelColumns(Gtk::ComboBox *Combo);
	protected:
		Gtk::TreeModelColumn<int> Id;
		Gtk::TreeModelColumn<Glib::ustring> Name;
		Glib::RefPtr<Gtk::ListStore> FuelQtyrefTreeModel;
	};
	FuelQtyModelColumns *FuelQtyComboColumns;

	class FuelTypeModelColumns : public Gtk::TreeModel::ColumnRecord {
	public:
		FuelTypeModelColumns(Gtk::ComboBox *Combo);
	protected:
		Gtk::TreeModelColumn<int> Id;
		Gtk::TreeModelColumn<Glib::ustring> Name;
		Glib::RefPtr<Gtk::ListStore> FuelTyperefTreeModel;
	};
	FuelTypeModelColumns *FuelTypeComboColumns;

private:
	std::string *GladeFilename;
	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Gtk::Dialog *GtkDialog;

	Gtk::Entry *DescEntry;
	Gtk::Entry *RegistrationEntry;

	Gtk::Entry *TASClimbEntry,*TASCruiseEntry, *TASDescentEntry;
	Gtk::Entry *ClimbRateEntry, *DescentRateEntry;
	Gtk::RadioButton *SegmentedRadioButton;
	Gtk::Entry *ClimbFuelRateEntry, *CruiseFuelRateEntry, *DescentFuelRateEntry;
	Gtk::RadioButton *HourlyRadioButton;
	Gtk::Entry *Hour1Entry,*Hour2Entry,*Hour3Entry,*Hour4Entry,*Hour5Entry;
	Gtk::Entry *Hour6Entry,*Hour7Entry,*Hour8Entry,*Hour9Entry,*AddlHoursEntry;

	Gtk::ComboBox *AirspeedUnitsCombo;
	Gtk::ComboBox *ClimbDescentRateUnitsCombo;
	Gtk::ComboBox *AltitudeUnitsCombo;
	Gtk::ComboBox *AircraftWeightUnitsCombo;
	Gtk::ComboBox *CGStationDistUnitsCombo;
	Gtk::ComboBox *FuelQtyUnitsCombo;
	Gtk::ComboBox *FuelTypeCombo;
	Gtk::ComboBox *OilQtyUnitsCombo;

	Aircraft *AircraftData;

	void on_SegmentedRadioButton_clicked();
	void on_HourlyRadioButton_clicked();
};

#endif /*AIRCRAFTDIALOG_H_*/
