#ifndef CHARTDRAWINGAREA_H_
#define CHARTDRAWINGAREA_H_

#include <libglademm.h>
#include <gtkmm/drawingarea.h>
#include <gtkmm/statusbar.h>
#include <gdkmm/pixbuf.h>
#include <gtkmm/window.h>
#include "GPS.h"
#include "SectionalChart.h"
#include "Route.h"
#include "Config.h"
#include "DataStore.h"
#include "Airport.h"
#include "Shapefile.h"

class ChartDrawingArea: public Gtk::DrawingArea
{
public:
	ChartDrawingArea(BaseObjectType* cobject,const Glib::RefPtr<Gnome::Glade::Xml>& glade_xml);
	~ChartDrawingArea();

	void Shift(int x, int y);
	void Slide(double x, double y);
	int GetXOffset();
	int GetYOffset();
	void SetOffset(int x, int y);
	void ProjectPixel(int *x, int *y);
	bool OpenSectional(std::string filename);
	void CloseChart();
	void SetStatusbar(Gtk::Statusbar *Statusbar);
	void Redraw();
	void PutPositionInStatusbar();
	void SetDisplayScale(double scale);
	double GetDispalyScale();
	void ScaleDisplayScale(double adjustment);
	void SetInterpType(Gdk::InterpType InterpType);
	void LonLatToDMMSSPSSSS(double Lon, double Lat, std::string *str);
	void LonLatToDMMPMMMMMM(double Lon, double Lat, std::string *str);
	void LonLatToDPDDDDDDD(double Lon, double Lat, std::string *str);
	void CenterOnLonLat(double Lon, double Lat);
	void MarkLocation(double Lon, double Lat, double *Trk=NULL, double *Spd=NULL, double *Alt=NULL);
	void ClearLocation();
	void DrawLocation();
	void DrawGroundspeed();
	void DrawGPSAltitude();
	void DrawTrack();
	std::string GetCurrSectionalFileName();
	SectionalChart *GetCurrSectionalChart();
	Glib::RefPtr<Gdk::Window> GetWindow();
	void DrawGreatCircleRoute(Route *RouteData);
	void DrawAirportsWx(Volume *VolData);
	void DrawAirportWx(Cairo::RefPtr<Cairo::Context> Context, Airport *AirportData);
	void DrawRhumbLineRoute(Route *RouteData);
	void SetConfig(Config *Cfg);
	void SetRoute(Route *RouteData);
	void SetVolData(Volume *VolData);
	void SetShapefile(Shapefile *ShapefileData);
//	bool LoadStateShapes();
//	void ClearStateShapes();
	void DrawShapes(GdkEventExpose* ev);

private:
	bool on_expose_event(GdkEventExpose* ev);
	bool on_motion_notify_event(GdkEventMotion* event);
	bool on_button_press_event(GdkEventButton * event);
	bool on_button_release_event(GdkEventButton * event);
	void on_Center_activate(int x, int y);

	static const int DataFieldValueSize=50;
	static const int DataFieldLabelSize=20;
	static const int DataFieldPadding=10;

	Gtk::Statusbar *Statusbar;
	bool Dragging;
	int DragStartX,DragStartY;
	void ClipOffset();
	void ClipOffset(int pixwidth, int pixheight, int viewwidth, int viewheight);
	double DisplayScale;
	int xoffset,yoffset;
	int OldWidth,OldHeight;
	Gdk::InterpType InterpType;
	Gtk::Menu *ContextMenu;
	Route *RouteData;

	time_t LonLatTime, VelTime, SpdTime, AltTime;
	double Lon, Lat, Trk, Spd, Alt;

	Config *Cfg;
	SectionalChart *Sectional;
	Volume *VolData;
	Shapefile *ShapefileData;
};

#endif /*CHARTDRAWINGAREA_H_*/
