#ifndef DATAMANAGERDIALOG_H_
#define DATAMANAGERDIALOG_H_

#include <gtkmm/dialog.h>
#include <gtkmm/entry.h>
#include <gtkmm/button.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <gtkmm/window.h>
#include <gtkmm/progressbar.h>
#include <libglademm.h>
#include <string>
#include "DataStore.h"
#include "GeoChart.h"
#include "Airport.h"
#include "Config.h"
#include "Shapefile.h"

class DataManagerDialogModelColumns : public Gtk::TreeModelColumnRecord
{
public:
	DataManagerDialogModelColumns();

	Gtk::TreeModelColumn<std::string> Active;
	Gtk::TreeModelColumn<std::string> VolumeName;
	Gtk::TreeModelColumn<std::string> EffFrom;
	Gtk::TreeModelColumn<std::string> EffTo;
	Gtk::TreeModelColumn<int> nAirports;
	Gtk::TreeModelColumn<int> nNavAids;
	Gtk::TreeModelColumn<int> nWaypoints;
	Gtk::TreeModelColumn<int> nCharts;
	Gtk::TreeModelColumn<int> nApproachPlates;
	Gtk::TreeModelColumn<std::list<Volume*>::iterator> Volumei;
};

class DataManagerDialog
{
public:
	DataManagerDialog(Config *Cfg, std::string *GladeFilename, DataStore *Data, Shapefile *ShapefileData);
	virtual ~DataManagerDialog();

	void Run();
private:
	Glib::RefPtr<Gnome::Glade::Xml> refXml,ProgRefXml;
	Gtk::Dialog *GtkDialog;
	Gtk::Button *ActivateButton;
	Gtk::Button *DeleteButton;
	Gtk::Button *ImportButton;
	Gtk::TreeView *DataManagerTreeView;

	Gtk::Button *CancelButton;
	Gtk::Window *ProgressWindow;
	Gtk::ProgressBar *ProgressBar,*SubProgressBar;

	DataManagerDialogModelColumns Columns;
	Glib::RefPtr<Gtk::ListStore> ListStore;

	Config *Cfg;
	std::string *GladeFilename;
	DataStore *Data;
	Shapefile *ShapefileData;
	bool TimeToClose;
	bool CancelFlag;

	void PopulateTreeView();
	bool DeleteVolume(std::list<Volume*>::iterator Volumei);
	bool DeleteGeoCharts(std::string Dir,std::vector<GeoChart*> *GeoCharts);
	bool DeleteApproachPlates(std::string Dir,std::vector<Airport*> *Airports);

	void on_ActivateButton_clicked();
	void on_DeleteButton_clicked();
	void on_ImportButton_clicked();
	void on_CancelButton_clicked();
};

#endif /*DATAMANAGERDIALOG_H_*/
