#include <gtkmm/main.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/drawingarea.h>
#include <gtkmm/window.h>
#include <gtkmm/accelgroup.h>
#include <cairo.h>

#include "../config.h"
#include "FlashlightDialog.h"

FlashlightDialog::FlashlightDialog(std::string *GladeFilename)
{
	this->GladeFilename=GladeFilename;

	try
	{
		refXml = Gnome::Glade::Xml::create(*GladeFilename, "FlashlightDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog("Error reading data file.  Program cannot continue.",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		delete message;
		Gtk::Main::quit();
	}

	refXml->get_widget("FlashlightDialog",GtkDialog);
	refXml->get_widget("RedRadioButton",RedRadioButton);
	refXml->get_widget("WhiteRadioButton",WhiteRadioButton);
	refXml->get_widget("BrightnessHScale",BrightnessHScale);
	refXml->get_widget("CloseButton",CloseButton);
	refXml->get_widget("BrightnessLabel",BrightnessLabel);
	refXml->get_widget("DrawingArea",DrawingArea);

	RedRadioButton->signal_clicked().connect(sigc::mem_fun(this,&FlashlightDialog::on_RedRadioButton_clicked));
	WhiteRadioButton->signal_clicked().connect(sigc::mem_fun(this,&FlashlightDialog::on_WhiteRadioButton_clicked));
	BrightnessHScale->signal_value_changed().connect(sigc::mem_fun(this,&FlashlightDialog::on_BrightnessHScale_value_changed));
	CloseButton->signal_clicked().connect(sigc::mem_fun(this,&FlashlightDialog::on_CloseButton_clicked));
	DrawingArea->signal_expose_event().connect(sigc::mem_fun(this,&FlashlightDialog::on_DrawingArea_expose_event));
	BrightnessLabel->signal_mnemonic_activate().connect(sigc::mem_fun(this,&FlashlightDialog::on_BrightnessLabel_mnemonic_activate));

	/* Ensure that we aren't blinding by default */
	RedRadioButton->set_active();
	BrightnessHScale->set_value(50.0);
	BrightnessHScale->grab_focus();

	GtkDialog->maximize();
	GtkDialog->fullscreen();
}

FlashlightDialog::~FlashlightDialog() {
	delete GtkDialog;
}

int FlashlightDialog::Run() {
	return GtkDialog->run();
}

void FlashlightDialog::on_RedRadioButton_clicked() {
	Gdk::Rectangle rect(0,0,DrawingArea->get_width(),DrawingArea->get_height());
	DrawingArea->get_window()->invalidate_rect(rect,1);
}

void FlashlightDialog::on_WhiteRadioButton_clicked() {
	Gdk::Rectangle rect(0,0,DrawingArea->get_width(),DrawingArea->get_height());
	DrawingArea->get_window()->invalidate_rect(rect,1);
}

void FlashlightDialog::on_BrightnessHScale_value_changed() {
	Gdk::Rectangle rect(0,0,DrawingArea->get_width(),DrawingArea->get_height());
	DrawingArea->get_window()->invalidate_rect(rect,1);
}

void FlashlightDialog::on_CloseButton_clicked() {
	GtkDialog->response(Gtk::RESPONSE_NONE);
}

bool FlashlightDialog::on_BrightnessLabel_mnemonic_activate(bool group_cycling) {
	BrightnessHScale->grab_focus();
	return true;
}

bool FlashlightDialog::on_DrawingArea_expose_event(GdkEventExpose* ev) {
	Cairo::RefPtr<Cairo::Context> Context = DrawingArea->get_window()->create_cairo_context();

	/* Set up brush */
	double Brightness=BrightnessHScale->get_value()/100.0;

	if (RedRadioButton->get_active()) {
		Context->set_source_rgb(Brightness, 0.0, 0.0);
	} else {
		Context->set_source_rgb(Brightness, Brightness, Brightness);
	}

	Context->rectangle(0,0,DrawingArea->get_width(),DrawingArea->get_height());
	Context->fill();
	Context->clip();

	Context->stroke();
	return true;
}

