#ifndef GEODESIC_H_
#define GEODESIC_H_

#define GEODESIC_WGS84_SEMI_MAJORAXIS 6378137.0
#define GEODESIC_WGS84_SEMI_MINORAXIS 6356752.3142

class Geodesic
{
	/* Find the distance (meters) and bearings between two Lon/Lat pairs (given in degrees)
	 * Results are in meters and degrees as appropriate 
	 */
public:
	static double GreatCircleDistBear(
		double Lon1, double Lat1,
		double Lon2, double Lat2,
		double *Dist=NULL, double *Bear1=NULL, double *Bear2=NULL);
	static void GreatCircleTravel(
		double Lon1, double Lat1,double Dist,double Bear1,
		double *Lon2=NULL, double *Lat2=NULL, double *Bear2=NULL);
		

	/* Geodesic parameters per WGS-84 */
	static const double a=GEODESIC_WGS84_SEMI_MAJORAXIS;		/* in meters */
	static const double b=GEODESIC_WGS84_SEMI_MINORAXIS;		/* in meters */
	static const double f=										/* Flattening */
		(GEODESIC_WGS84_SEMI_MAJORAXIS-GEODESIC_WGS84_SEMI_MINORAXIS)/
		GEODESIC_WGS84_SEMI_MAJORAXIS;
};

#endif /*GEODESIC_H_*/
