#include <gtkmm/messagedialog.h>
#include <gtkmm/main.h>
#include "ImportDialog.h"
#include "Exception.h"
#include "ImportdTPPDVD.h"
#include "ImportSectionalsDVD.h"
#include "ImportDigitalAeroInformation.h"
#include "ImportNationalFlightDatabase.h"
#include "ImportShapefile.h"

ImportDialog::ImportDialog(Config *Cfg, std::string *GladeFilename, DataStore *Data, Shapefile *ShapefileData)
{
	try
	{
		refXml = Gnome::Glade::Xml::create(*GladeFilename, "ImportDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog("Error reading data file.  Program cannot continue.",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
	}

	refXml->get_widget("ImportDialog",GtkDialog);
	refXml->get_widget("FlightDBDVDButton",FlightDBDVDButton);
	refXml->get_widget("SectionalsDVDButton",SectionalsDVDButton);
	refXml->get_widget("ApproachesDVDButton",ApproachesDVDButton);
	refXml->get_widget("ShapefileButton",ShapefileButton);

	FlightDBDVDButton->signal_clicked().connect(sigc::mem_fun(this,&ImportDialog::on_FlightDBDVDButton_clicked));
	SectionalsDVDButton->signal_clicked().connect(sigc::mem_fun(this,&ImportDialog::on_SectionalsDVDButton_clicked));
	ApproachesDVDButton->signal_clicked().connect(sigc::mem_fun(this,&ImportDialog::on_ApproachesDVDButton_clicked));
	ShapefileButton->signal_clicked().connect(sigc::mem_fun(this,&ImportDialog::on_ShapefileButton_clicked));

	this->Cfg=Cfg;
	this->GladeFilename=GladeFilename;
	this->Data=Data;
	this->ShapefileData=ShapefileData;
}

ImportDialog::~ImportDialog()
{
	delete GtkDialog;
}

void ImportDialog::Run() {
	GtkDialog->run();
}

void ImportDialog::on_FlightDBDVDButton_clicked() {
	try {
		ImportNationalFlightDatabase Import(Cfg,GladeFilename,Data);
		Import.Run();
	}
	catch (Exception Ex) {
		Ex.Show();
	}
}

void ImportDialog::on_SectionalsDVDButton_clicked() {
	try {
		ImportSectionalsDVD Import(Cfg,GladeFilename,Data);
		Import.Run();
	}
	catch (Exception Ex) {
		Ex.Show();
	}
}

void ImportDialog::on_ApproachesDVDButton_clicked() {
	try {
		ImportdTPPDVD Import(Cfg,GladeFilename,Data);
		Import.Run();
	}
	catch (Exception Ex) {
		Ex.Show();
	}
}

void ImportDialog::on_ShapefileButton_clicked() {
	try {
		ImportShapefile Import(Cfg,GladeFilename,ShapefileData);
		Import.Run();
	}
	catch (Exception Ex) {
		Ex.Show();
	}
}
