#ifndef METAR_H_
#define METAR_H_

#include <time.h>
#include <string>

class METAR
{
public:

	enum CeilingCover_enum {
		CEILINGCOVER_MISSING,	/* M */
		CEILINGCOVER_CLEAR,		/* CLR - Clear below 12,000 */
		CEILINGCOVER_SKYCLEAR,	/* SKC - Completely clear */
		CEILINGCOVER_FEW,		/* 1/8 - 2/8 */
		CEILINGCOVER_SCATTERED,	/* 3/8 - 4/8 */
		CEILINGCOVER_BROKEN,	/* 5/8 - 7/8 */
		CEILINGCOVER_OVERCAST,	/* 8/8 */
		CEILINGCOVER_OBSCURED,
		CEILINGCOVER_INVALID
	};

	enum FR_enum {
		FR_VFR,
		FR_MVFR,
		FR_IFR,
		FR_LIFR,
		FR_INVALID
	};

	METAR();
	METAR(std::string Text);
	virtual ~METAR();

	bool Set(std::string Text);
	std::string GetText();
	std::string GetStation();
	bool GetValidParseFlag();
	time_t GetZuluTime();
	double GetVisibility();
	int GetCeiling();
	enum CeilingCover_enum GetCeilingCover();
	enum FR_enum GetFR();

	static const char WxCodeSize=3;	/* Enough to hold 2 chars and a terminating zero */
	static const char WxCodes[][WxCodeSize];

private:
	bool ValidParseFlag;
	std::string Text;
	std::string Station;
	time_t ZuluTime;
	double Visibility;
	int Ceiling;
	enum CeilingCover_enum CeilingCover;

	bool Parse();
};

bool operator < (const METAR& left, const METAR& right);
bool operator == (const METAR& left, const METAR& right);

#endif /*METAR_H_*/
