#ifndef MAINWINDOW_H_
#define MAINWINDOW_H_

#include <time.h>
#include <string>
#include <list>
#include <gtkmm/window.h>
#include <gtkmm/menuitem.h>
#include <gtkmm/menu.h>
#include <gtkmm/statusbar.h>
#include <libglademm.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>

#include "vifir.h"
#include "ChartDrawingArea.h"
#include "GPS.h"
#include "Config.h"
#include "RouteDialog.h"
#include "Route.h"
#include "DataStore.h"
#include "DataManagerDialog.h"
#include "Aircraft.h"
#include "METARs.h"
#include "Shapefile.h"

#define MAINWINDOW_DBUS_SS_SERVICE   "org.gnome.ScreenSaver"
#define MAINWINDOW_DBUS_SS_PATH      "/org/gnome/ScreenSaver"
#define MAINWINDOW_DBUS_SS_INTERFACE "org.gnome.ScreenSaver"

#define MAINWINDOW_DBUS_PM_SERVICE		"org.freedesktop.PowerManagement"
#define MAINWINDOW_DBUS_PM_PATH			"/org/freedesktop/PowerManagement/Inhibit"
#define MAINWINDOW_DBUS_PM_INTERFACE	"org.freedesktop.PowerManagement.Inhibit"

class MainWindow
{
public:
	MainWindow(App *Application, std::string *GladeFilename, Config *Cfg);
	virtual ~MainWindow();

	Gtk::Window *GetGtkWindow() {return GtkWindow;}

private:
	Config *Cfg;

	void on_FileNewAircraft_activate();
	void on_FileNewRoute_activate();
	void on_FileNewWxData_activate();
	void on_FileOpenAircraft_activate();
	void on_FileOpenChart_activate();
	void on_FileOpenRoute_activate();
	void on_FileOpenWxData_activate();
	void on_FileClose_activate();
	void on_FileDataManager_activate();
	void on_FileConfigure_activate();
	void on_FileFlashlight_activate();
	void on_FileQuit_activate();
	void on_ViewZoomIn_activate();
	void on_ViewZoomOut_activate();
	void on_ViewZoom30Pct_activate();
	void on_ViewZoom60Pct_activate();
	void on_ViewZoom100Pct_activate();
	void on_ViewZoom150Pct_activate();
	void on_ViewZoom200Pct_activate();
	void on_ViewZoom400Pct_activate();
	void on_ViewPanLeft_activate();
	void on_ViewPanRight_activate();
	void on_ViewPanUp_activate();
	void on_ViewPanDown_activate();
	void on_ViewPanFastLeft_activate();
	void on_ViewPanFastRight_activate();
	void on_ViewPanFastUp_activate();
	void on_ViewPanFastDown_activate();
	void on_ViewGoto_activate();
	void on_ViewFullscreen_activate();

	void on_EditAircraft_activate();
	void on_EditRoute_activate();

	void on_SearchApproaches_activate();

	void on_GPSActive_activate();
	void on_GPSStartgpsd_activate();

	void on_HelpAbout_activate();
	void on_HelpDisclaimer_activate();

	void on_CancelButton_clicked();

	void SetRecentFilesMenu();
	void OpenGeoChart(std::string filename);
	bool GPSHandler(int timer_number);
	bool SetGPSActive(bool value=true);
	bool Startgpsd();
	void LinkMETARsToAirports(std::vector<Airport*> *Airports, METARs *METARData);

	bool DBusInit();
	bool DBusShutdown();
	bool InhibitSS();
	bool UnInhibitSS();
	bool InhibitPM();
	bool UnInhibitPM();

	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Gtk::Window *GtkWindow;
	Gtk::MenuItem *FileNewAircraft;
	Gtk::MenuItem *FileNewRoute;
	Gtk::MenuItem *FileNewWxData;
	Gtk::MenuItem *FileOpenAircraft;
	Gtk::MenuItem *FileOpenChart;
	Gtk::MenuItem *FileOpenRoute;
	Gtk::MenuItem *FileOpenWxData;
	Gtk::MenuItem *FileClose;
	Gtk::MenuItem *FileDataManager;
	Gtk::MenuItem *FileConfigure;
	Gtk::MenuItem *FileFlashlight;
	Gtk::MenuItem *FileQuit;
	Gtk::MenuItem *EditAircraft;
	Gtk::MenuItem *EditRoute;
	Gtk::MenuItem *ViewMenu;
	Gtk::MenuItem *ViewZoomIn;
	Gtk::MenuItem *ViewZoomOut;
	Gtk::MenuItem *ViewZoom30Pct;
	Gtk::MenuItem *ViewZoom60Pct;
	Gtk::MenuItem *ViewZoom100Pct;
	Gtk::MenuItem *ViewZoom150Pct;
	Gtk::MenuItem *ViewZoom200Pct;
	Gtk::MenuItem *ViewZoom400Pct;
	Gtk::MenuItem *ViewPanLeft;
	Gtk::MenuItem *ViewPanRight;
	Gtk::MenuItem *ViewPanUp;
	Gtk::MenuItem *ViewPanDown;
	Gtk::MenuItem *ViewPanFastLeft;
	Gtk::MenuItem *ViewPanFastRight;
	Gtk::MenuItem *ViewPanFastUp;
	Gtk::MenuItem *ViewPanFastDown;
	Gtk::MenuItem *ViewGoto;
	Gtk::CheckMenuItem *ViewFullscreen;

	Gtk::MenuItem *SearchApproaches;

	Gtk::CheckMenuItem *GPSActive;
	Gtk::MenuItem *GPSStartgpsd;
	Gtk::CheckMenuItem *GPSAutoSelectChart;

	Gtk::MenuItem *HelpDisclaimer;
	Gtk::MenuItem *HelpAbout;
	Gtk::Statusbar *Statusbar;
	ChartDrawingArea *DrawingArea;
	App *Application;
	std::string *GladeFilename;
	GPS *Gps;
	sigc::connection GPSHandlerConnection;
	time_t LastSectionalChange;
	Route RouteData;
	Aircraft *AircraftData;
	DataStore *Data;
	Shapefile *ShapefileData;
	bool CancelFlag;

	DBusGProxy *DBusInhibitSSProxy,*DBusInhibitPMProxy;
	guint32 *DBusInhibitSSCookie,*DBusInhibitPMCookie;
};

#endif /*MAINWINDOW_H_*/
