#define __STDC_LIMIT_MACROS
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "NavAid.h"
#include "ARINC424_18.h"

NavAid::NavAid()
{
	LocType=Waypoint::LOC_NAVAID;
	Declination=0.0;
}

NavAid::~NavAid()
{
}

Frequency *NavAid::GetFreq()
{
	return &Freq;
}

void NavAid::SetFreq(Frequency *NewFreq)
{
	Freq.Clone(NewFreq);
}

bool NavAid::ParseNFD(std::string Line)
{
	struct arinc424_18_navaid_header_struct *NFDHeader;
	char strbuf[sizeof(struct arinc424_18_navaid_header_struct)];

	if (Line.length()!=sizeof(struct arinc424_18_vhfnavaid_struct)+1)	/* Allow for LF - all the lines are the same length */
	{
		return false;
	}

	if (
		strcmp(arinc424_18_parse_recordtype(Line.c_str()),ARINC424_18_RECTYPE_NAVAID_VHF)!=0 &&
		strcmp(arinc424_18_parse_recordtype(Line.c_str()),ARINC424_18_RECTYPE_NAVAID_NDB)!=0
		)
	{
		return false;
	}
	NFDHeader=(struct arinc424_18_navaid_header_struct *)Line.c_str();

	/* ID */
	memcpy(strbuf,NFDHeader->id,sizeof(NFDHeader->id));
	strbuf[sizeof(NFDHeader->id)]=0;

	/* Trim trailing spaces */
	while (isspace(strbuf[strlen(strbuf)-1]) && strlen(strbuf))
		strbuf[strlen(strbuf)-1]=0;
	ID=strbuf;

	return true;
}

VOR::VOR()
{
}

enum NavAid::Type_enum VOR::GetType() {
	return TYPE_VOR;
}

bool VOR::ParseNFD(std::string Line)
{
	arinc424_18_vhfnavaid_struct *NFDData;
	char strbuf[sizeof(struct arinc424_18_airport_struct)];

	if (!NavAid::ParseNFD(Line))
	{
		return false;
	}

	NFDData=(struct arinc424_18_vhfnavaid_struct *)Line.c_str();

	/* Lat/Lon */
	Lat=arinc424_18_parse_lat(NFDData->lat);
	Lon=arinc424_18_parse_lon(NFDData->lon);
	if (isnan(Lat) || isnan(Lon))
	{
		return false;
	}

	/* Frequency */
	Freq.SetRawFreq(arinc424_18_parse_freq(NFDData->frequency));
	Freq.SetPrecision(3);
	Freq.SetUnits(Frequency::MHZ);
	Freq.SetType(Frequency::VOR);

	/* Declination */
	memcpy(strbuf,NFDData->declination,sizeof(NFDData->declination));
	strbuf[sizeof(NFDData->declination)]=0;
	Declination=(atoi(&(strbuf[1])))/10.0;
	if (strbuf[0]=='W')
		Declination=-Declination;

	/* Name */
	memcpy(strbuf,NFDData->name,sizeof(NFDData->name));
	strbuf[sizeof(NFDData->name)]=0;
	/* Trim trailing spaces */
	while (isspace(strbuf[strlen(strbuf)-1]) && strlen(strbuf))
		strbuf[strlen(strbuf)-1]=0;
	Name=strbuf;

	return true;
}

NDB::NDB()
{
}

enum NavAid::Type_enum NDB::GetType() {
	return TYPE_NDB;
}

bool NDB::ParseNFD(std::string Line)
{
	arinc424_18_ndbnavaid_struct *NFDData;
	char strbuf[sizeof(struct arinc424_18_airport_struct)];
	if (!NavAid::ParseNFD(Line)) {
		return false;
	}

	NFDData=(struct arinc424_18_ndbnavaid_struct *)Line.c_str();

	Lat=arinc424_18_parse_lat(NFDData->lat);
	Lon=arinc424_18_parse_lon(NFDData->lon);
	if (isnan(Lat) || isnan(Lon))
	{
		return false;
	}

	/* Frequency */
	Freq.SetRawFreq(arinc424_18_parse_freq(NFDData->frequency));
	Freq.SetPrecision(3);
	Freq.SetUnits(Frequency::MHZ);
	Freq.SetType(Frequency::NDB);

	/* Name */
	memcpy(strbuf,NFDData->name,sizeof(NFDData->name));
	strbuf[sizeof(NFDData->name)]=0;
	/* Trim trailing spaces */
	while (isspace(strbuf[strlen(strbuf)-1]) && strlen(strbuf))
		strbuf[strlen(strbuf)-1]=0;
	Name=strbuf;

	return true;
}

ILS::ILS()
{
}

enum NavAid::Type_enum ILS::GetType() {
	return TYPE_ILS;
}

Localizer::Localizer()
{
}

enum NavAid::Type_enum Localizer::GetType() {
	return TYPE_LOCALIZER;
}

bool NavAid::MakeBinary(struct NavAid_Binary_struct *BinaryData) {
	double Lonx1000000,Latx1000000,Decx100;

	Lonx1000000=Lon*1000000.0;
	Latx1000000=Lat*1000000.0;
	Decx100=Declination*100.0;
	/* Sanity checking */
	if (
		ID.size()>sizeof(BinaryData->ID) ||
		Lonx1000000<INT32_MIN || Lonx1000000>INT32_MAX ||
		Latx1000000<INT32_MIN || Latx1000000>INT32_MAX ||
		Decx100>INT32_MAX || Decx100<INT32_MIN ||
		LocType>INT8_MAX ||
		Name.size()>sizeof(BinaryData->Name)
		) {
		return false;
	}

	strncpy(BinaryData->ID,ID.c_str(),sizeof(BinaryData->ID));
	BinaryData->Lonx1000000=(int32_t)round(Lonx1000000);
	BinaryData->Latx1000000=(int32_t)round(Latx1000000);
	BinaryData->Declinationx100=(int32_t)round(Decx100);
	BinaryData->Type=GetType();
	strncpy(BinaryData->Name,Name.c_str(),sizeof(BinaryData->Name));
	return true;
}

void NavAid::SetFromBinary(struct NavAid_Binary_struct *BinaryData) {
	ID.assign(BinaryData->ID,sizeof(BinaryData->ID));
	Lon=BinaryData->Lonx1000000/1000000.0;
	Lat=BinaryData->Latx1000000/1000000.0;
	Declination=BinaryData->Declinationx100/100.0;
	Name.assign(BinaryData->Name,sizeof(BinaryData->Name));
	/* Reset the size to the actual number of characters */
	ID=ID.c_str();
	Name=Name.c_str();

	/* The Type is set when the class is instantiated */
}
