#ifndef ROUTE_H_
#define ROUTE_H_

#include <list>
#include <gtkmm/window.h>
#include "RoutePoint.h"
#include "SectionalChart.h"
#include "CRC32.h"

#define ROUTE_FILEMAGIC "Vifir_Route"

class Route
{
public:
	Route();
	void Add(RoutePoint *RP);
	void Del(std::list<RoutePoint*>::iterator RPi);
	void Shift(std::list<RoutePoint*>::iterator RPi, int Direction);
	void Reverse();
	std::list<RoutePoint*> *GetRoutePoints();
	std::list<RoutePoint*> *GetLinearRoutePoints();
	bool MakeBinary(struct Route_Binary_struct *BinaryData);
	void SetFromBinary(struct Route_Binary_struct *BinaryData);
	bool Load(std::string Filename);
	bool Load();
	void Save(std::string Filename);
	void Save();
	bool GetChanged();
	void Clear();
	void ClearLinRoutePoints();

private:
	void ComputeLinRoutePoints();
	void LoadRoutePoint(std::list<RoutePoint*> *RoutePointV,std::ifstream *ifs, uint32_t *RunningCRC);
	void SaveRoutePoints(std::list<RoutePoint*> *RoutePointV, std::ofstream *ofs, uint32_t *RunningSize, uint32_t *RunningCRC);

	std::list<RoutePoint*> RoutePoints;
	std::list<RoutePoint*> LinRoutePoints;
	CRC32 CRC;
	std::string Filename;
};

/* Increment the version when the data structure changes */
#define ROUTE_HEADER_STRUCT_VER 1
struct Route_Header_struct {
	char Magic[sizeof(ROUTE_FILEMAGIC)];
	uint32_t DataVersion;
	uint32_t FileSize;
	uint32_t CRC32;	/* Computed with this field and FileSize set to zero */
	uint32_t nRoutePoints;
} __attribute__ ((packed));

/* Increment the version when the structure changes */
#define ROUTE_BINARY_STRUCT_VER 1
struct Route_Binary_struct {
} __attribute__ ((packed));

#endif /*ROUTE_H_*/
