#ifndef SECTIONALCHART_H_
#define SECTIONALCHART_H_

#include <time.h>
#include <string>
#include <gdkmm/pixbuf.h>
#include "tiffio.h"
#include "geotiff.h"
#include "geo_normalize.h"
#include "Config.h"

class SectionalChart
{
public:
	SectionalChart(Config *Cfg,std::string FileName);
	virtual ~SectionalChart();

	Glib::RefPtr<Gdk::Pixbuf> GetPixbuf();

	bool LoadRaster();
	void PixelToLonLat(int XPixel, int YPixel, double *Lon, double *Lan);
	void PixelToMeters(int XPixel, int YPixel, double *XMeters, double *YMeters);
	void MetersToLonLat(double XMeters, double YMeters, double *Lon, double *Lat);

	void LonLatToPixel(double Lon, double Lat, int *XPixel, int *YPixel);
	void MetersToPixel(double XMeters, double YMeters, int *XPixel, int *YPixel);
	void LonLatToMeters(double Lon, double Lat, double *XMeters, double *YMeters);
	bool GetValid();
	bool GetBounds(double *East, double *West, double *North, double *South);
	bool GetRasterSize(uint32 *Width, uint32 *Height);
	bool GetRasterWidth(uint32 *w);
	bool GetRasterHeight(uint32 *h);
	bool GetGeoReferenced();
	std::string GetFileName();

private:
	uint32* chartraster;
	GTIF *geotif;
	TIFF *tif;
	GTIFDefn Defn;	/* Projection definition for proj */
	double XMetersPerPixel,YMetersPerPixel;
	double EastingOffset,NorthingOffset;
	double DisplayScale;
	bool GeoReferenced;
	std::string FileName;

	Glib::RefPtr<Gdk::Pixbuf> pixbuf;
};

#endif /*SECTIONALCHART_H_*/
