#define __STDC_LIMIT_MACROS
#include <stdint.h>
#include <string.h>
#include "Waypoint.h"
#include "Util.h"

Waypoint::Waypoint()
{
	Lon=NAN;
	Lat=NAN;
	LocType=LOC_UNKNOWN;
	ID="";
	Name="";
}

std::string Waypoint::GetID()
{
	return ID;
}

double Waypoint::GetLon()
{
	return Lon;
}

double Waypoint::GetLat()
{
	return Lat;
}

void Waypoint::SetLonLat(double Lon, double Lat)
{
	this->Lon=Lon;
	this->Lat=Lat;
}

bool Waypoint::Matchi(std::string Str)
{
	Str=Util::toUpper(Str);
	if (ID.find(Str)!=Str.npos)
		return true;

	if (Name.find(Str)!=Str.npos)
		return true;

	return false;
}

enum Waypoint::LocType_enum Waypoint::GetLocType()
{
	return LocType;
}

std::string Waypoint::GetLocTypeStr(enum LocType_enum LocType)
{
	const char LocType_str[][20]={"Unknown","Airport","NavAid","EnRoute Waypoint"};

	if (LocType<=3)
		return LocType_str[LocType];

	return "Unknown";
}

/* Return our location type string if none explicitly specified */
std::string Waypoint::GetLocTypeStr()
{
	return GetLocTypeStr(this->LocType);
}

std::string Waypoint::GetName()
{
	return Name;
}

bool Waypoint::MakeBinary(struct Waypoint_Binary_struct *BinaryData) {
	double Lonx1000000,Latx1000000;

	Lonx1000000=Lon*1000000.0;
	Latx1000000=Lat*1000000.0;
	/* Sanity checking */
	if (
		ID.size()>sizeof(BinaryData->ID) ||
		Name.size()>sizeof(BinaryData->Name) ||
		Lonx1000000<INT32_MIN || Lonx1000000>INT32_MAX ||
		Latx1000000<INT32_MIN || Latx1000000>INT32_MAX ||
		LocType>UINT8_MAX
		) {
		return false;
	}

	strncpy(BinaryData->ID,ID.c_str(),sizeof(BinaryData->ID));
	strncpy(BinaryData->Name,Name.c_str(),sizeof(BinaryData->Name));
	BinaryData->Lonx1000000=(int32_t)round(Lonx1000000);
	BinaryData->Latx1000000=(int32_t)round(Latx1000000);
	BinaryData->LocType=LocType;

	return true;
}

void Waypoint::SetFromBinary(struct Waypoint_Binary_struct *BinaryData) {
	ID.assign(BinaryData->ID,sizeof(BinaryData->ID));
	Name.assign(BinaryData->Name,sizeof(BinaryData->Name));
	Lon=BinaryData->Lonx1000000/1000000.0;
	Lat=BinaryData->Latx1000000/1000000.0;
	LocType=(enum LocType_enum)BinaryData->LocType;

	/* Reset the size to the actual number of characters */
	ID=ID.c_str();
	Name=Name.c_str();
}
