#include <gtkmm/messagedialog.h>
#include <gtkmm/main.h>
#include "WaypointSearchDialog.h"

WaypointSearchDialog::WaypointSearchDialog(std::string *GladeFilename, DataStore *Data)
{
	try
	{
		std::string gladefile;

		refXml = Gnome::Glade::Xml::create(*GladeFilename, "WaypointSearchDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog("Error reading data file.  Program cannot continue.",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
	}

	refXml->get_widget("WaypointSearchDialog",GtkDialog);
	refXml->get_widget("WaypointsTreeView",WaypointsTreeView);
	refXml->get_widget("NameEntry",NameEntry);
	refXml->get_widget("OKButton",OKButton);
	refXml->get_widget("CancelButton",CancelButton);
	refXml->get_widget("OnScreenKeypadParent",OnScreenKeypadParent);

	Keypad=new OnScreenKeypad();
	OnScreenKeypadParent->add(*Keypad);
	OnScreenKeypadParent->reorder_child(*Keypad,1);
	Keypad->show();
	Keypad->set_target(NameEntry);

	NameEntry->signal_changed().connect(sigc::mem_fun(this,&WaypointSearchDialog::on_NameEntry_changed));
	OKButton->signal_clicked().connect(sigc::mem_fun(this,&WaypointSearchDialog::on_OKButton_clicked));
	CancelButton->signal_clicked().connect(sigc::mem_fun(this,&WaypointSearchDialog::on_CancelButton_clicked));

	this->Data=Data;
	SearchList.clear();

	ListStore = Gtk::ListStore::create(Columns);
	WaypointsTreeView->set_model(ListStore);
	WaypointsTreeView->append_column("ID", Columns.ID);
	WaypointsTreeView->append_column("Type", Columns.LocType);
	WaypointsTreeView->append_column("Name", Columns.Name);
	WaypointsTreeView->signal_row_activated().connect(sigc::mem_fun(*this,&WaypointSearchDialog::on_Row_activated));
	/* Don't display the Waypoint column */

	Gtk::TreeView::Column* pColumn = WaypointsTreeView->get_column(0);
	if(pColumn)
		pColumn->set_sort_column(Columns.ID);

	pColumn = WaypointsTreeView->get_column(1);
	if(pColumn)
		pColumn->set_sort_column(Columns.LocType);

	pColumn = WaypointsTreeView->get_column(2);
	if(pColumn)
		pColumn->set_sort_column(Columns.Name);

	Selection=NULL;
}

WaypointSearchDialog::~WaypointSearchDialog() {
	delete GtkDialog;
}

int WaypointSearchDialog::Run() {
	int result;

	result=GtkDialog->run();
	if (result) {
	}

	return result;
}

void WaypointSearchDialog::on_OKButton_clicked() {
	Glib::RefPtr<Gtk::TreeSelection> WaypointSelection = WaypointsTreeView->get_selection();
	Gtk::TreeModel::iterator iter = WaypointSelection->get_selected();
	if (iter) {		/* If anything is selected */
		Gtk::TreeModel::Row row = *iter;
		Selection=row[Columns.WaypointData];
	}

	GtkDialog->response(Gtk::RESPONSE_OK);
}

void WaypointSearchDialog::on_CancelButton_clicked() {
	GtkDialog->response(Gtk::RESPONSE_CANCEL);
}

void WaypointSearchDialog::on_NameEntry_changed() {
	static std::string OldText,CurrText;
	std::vector<Waypoint*>::iterator Waypointi;
	CurrText=NameEntry->get_text();

	if (CurrText.length()<=1 || (CurrText.length()<=2 && (toupper(CurrText[0])=='K'))) {
		/* Don't search on a single character, or on two characters if the first one is a K (likely airport) */
		SearchList.clear();
		OldText=CurrText;
	} else {
		if (CurrText.substr(0,CurrText.length()-1)==OldText && (SearchList.size()>0 && CurrText.length()>2)) {
			/* The new search is a subset of the old one */
			std::vector<Waypoint*> NewSearchList;
			for (Waypointi=SearchList.begin();Waypointi!=SearchList.end();Waypointi++) {
				if ((*Waypointi)->Matchi(CurrText)) {
					NewSearchList.push_back(*Waypointi);
				}
			}
			SearchList=NewSearchList;
		} else {
			/* We have to search all of the waypoints */
			SearchList.clear();
			Data->WaypointSearch(CurrText,&SearchList);
		}
	}

	/* Rebuild the table */
	ListStore->clear();
	if (SearchList.size()<VOLUME_MAXSEARCHRESULTS) {
		for (Waypointi=SearchList.begin();Waypointi!=SearchList.end();Waypointi++) {
			Gtk::TreeModel::iterator iter = ListStore->append();
			Gtk::TreeModel::Row row = *iter;

			row[Columns.ID] = (*Waypointi)->GetID();
			row[Columns.LocType] = (*Waypointi)->GetLocTypeStr();
			row[Columns.Name] = (*Waypointi)->GetName();
			row[Columns.WaypointData] = (*Waypointi);
		}
	}

	if (SearchList.size()) {
		/* Select the first item by default */
		Glib::RefPtr<Gtk::TreeSelection> WaypointSelection = WaypointsTreeView->get_selection();
		Gtk::TreeModel::iterator iter = ListStore->children().begin();
		if(iter)
			WaypointSelection->select(iter);
	}

	OldText=CurrText;
}

void WaypointSearchDialog::on_Row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column) {
	on_OKButton_clicked();
}

WaypointSearchDialogModelColumns::WaypointSearchDialogModelColumns() {
	add(ID);
	add(LocType);
	add(Name);
	add(WaypointData);
}

Waypoint* WaypointSearchDialog::GetSelection() {
	return Selection;
}
